#
# Copyright (C) 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# Ref. python -c "import toml; help(toml); ..."
#
# pylint: disable=unused-argument
"""TOML backend.

.. versionadded:: 0.1.0

- Format to support: TOML, https://github.com/toml-lang/toml
- Requirements: (python) toml module, https://github.com/uiri/toml
- Limitations: None obvious
- Special options:

  - toml.load{s,} only accept '_dict' keyword option but it's used already to
    pass callable to make a container object.
"""
from __future__ import absolute_import

import toml
import anyconfig.backend.base
from anyconfig.backend.base import to_method


class Parser(anyconfig.backend.base.FromStreamLoader,
             anyconfig.backend.base.ToStreamDumper):
    """
    TOML parser.
    """
    _type = "toml"
    _extensions = ["toml"]

    dump_to_string = to_method(toml.dumps)
    dump_to_stream = to_method(toml.dump)

    def load_from_string(self, content, to_container, **opts):
        """
        Load TOML config from given string `content`.

        :param content: TOML config content
        :param to_container: callble to make a container object
        :param opts: keyword options passed to `toml.loads`

        :return: Dict-like object holding configuration
        """
        return toml.loads(content, _dict=to_container, **opts)

    def load_from_stream(self, stream, to_container, **opts):
        """
        Load TOML config from given stream `stream`.

        :param stream: Stream will provide config content string
        :param to_container: callble to make a container object
        :param opts: keyword options passed to `toml.load`

        :return: Dict-like object holding configuration
        """
        return toml.load(stream, _dict=to_container, **opts)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Red Hat, Inc.
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato redhat.com>
# License: MIT
#
# pylint: disable=import-error,unused-import
"""Module to keep backward compatibilities.

- basic str types are different in python 2.x and 3.x.
- collections.ordereddict is not available in python 2.6.
"""
from __future__ import absolute_import

try:
    from collections import OrderedDict  # flake8: noqa
except ImportError:
    from ordereddict import OrderedDict  # flake8: noqa

try:
    STR_TYPES = (basestring, unicode)  # flake8: noqa
except NameError:
    STR_TYPES = (str, )  # flake8: noqa

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# Ref. python -c "import msgpack; help(msgpack.Unpacker); help(msgpack.Packer)"
#
"""MessagePack file backend.

.. versionadded:: 0.0.11

- Format to support: MessagePack, http://msgpack.org
- Requirements: msgpack-python (https://pypi.python.org/pypi/msgpack-python)
- Limitations: None obvious
- Special options:

  - All options of msgpack.load{s,} and msgpack.dump{s,} except object_hook
    and file_like should work.

  - See also: http://pythonhosted.org/msgpack-python/api.html
"""
from __future__ import absolute_import

import msgpack
import anyconfig.backend.base
from anyconfig.backend.base import to_method


class Parser(anyconfig.backend.base.FromStreamLoader,
             anyconfig.backend.base.ToStreamDumper):
    """
    Loader/Dumper for MessagePack files.
    """
    _type = "msgpack"
    _extensions = []
    _load_opts = ["read_size", "use_list", "object_pairs_hook", "list_hook",
                  "encoding", "unicode_errors", "max_buffer_size", "ext_hook",
                  "max_str_len", "max_bin_len", "max_array_len", "max_map_len",
                  "max_ext_len", "object_pairs_hook"]
    _dump_opts = ["default", "encoding", "unicode_errors", "use_single_float",
                  "autoreset", "use_bin_type"]
    _open_flags = ('rb', 'wb')

    dump_to_string = to_method(msgpack.packb)
    dump_to_stream = to_method(msgpack.pack)

    def load_from_string(self, content, to_container, **opts):
        """
        Load config from given (byte) string `content`.

        :param content: MessagePack-ed config content
        :param to_container: callble to make a container object
        :param opts: keyword options passed to `msgpack.unpackb`

        :return: Dict-like object holding configuration
        """
        return msgpack.unpackb(content, object_hook=to_container, **opts)

    def load_from_stream(self, stream, to_container, **opts):
        """
        Load JSON config from given stream `stream`.

        :param stream: Stream will provide MessagePack-ed config content string
        :param to_container: callble to make a container object
        :param opts: keyword options passed to `msgpack.unpack`

        :return: Dict-like object holding configuration
        """
        return msgpack.unpack(stream, object_hook=to_container, **opts)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2012 - 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=missing-docstring
from __future__ import absolute_import
import unittest
import anyconfig.backend.properties as TT
import anyconfig.backend.tests.ini

from anyconfig.compat import OrderedDict as ODict


CNF_S = """
a = 0
  b = bbb
c:

sect0.c = x;y;z
sect1.d = \\
    1,2,3
"""
CNF = ODict((("a", "0"), ("b", "bbb"), ("c", ""),
             ("sect0.c", "x;y;z"), ("sect1.d", "1,2,3")))


class Test00(unittest.TestCase):

    def test_10_unescape(self):
        exp = "aaa:bbb"
        res = TT.unescape(r"aaa\:bbb")
        self.assertEqual(res, exp, res)

    def test_12_unescape(self):
        exp = r"\a"
        res = TT.unescape(r"\\a")
        self.assertEqual(res, exp, res)

    def test_20_escape(self):
        exp = r"\:\=\\ "
        res = TT.escape(r":=\ ")
        self.assertEqual(res, exp, res)


class Test10(anyconfig.backend.tests.ini.Test10):

    cnf = CNF
    cnf_s = CNF_S
    load_options = dict(comment_markers=("//", "#", "!"))
    dump_options = dict(dummy_opt="this_will_be_ignored")

    def setUp(self):
        self.psr = TT.Parser()


class Test20(anyconfig.backend.tests.ini.Test20):

    psr_cls = TT.Parser
    cnf = CNF
    cnf_s = CNF_S
    cnf_fn = "conf.properties"

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato @ redhat.com>
#
# pylint: disable=missing-docstring,invalid-name
from __future__ import absolute_import
import unittest
import m9dicts.dicts as TT


class Test_10_UpdateWithReplaceDict(unittest.TestCase):

    od0 = TT.OrderedDict((("a", 1), ("b", [1, 3]), ("c", "abc"), ("f", None)))
    up0 = TT.OrderedDict((("a", 2), ("b", [0, 1]),
                          ("c", TT.OrderedDict((("d", "d"), ("e", 1)))),
                          ("d", "d")))
    cls = TT.UpdateWithReplaceDict

    def setUp(self):
        self.mds = (self.cls(self.od0), self.cls(self.od0.items()),
                    self.cls(**self.od0))

    def check_updated(self, *others, **another):
        upd = self.up0
        ref = self.mds[0].copy()

        for md in self.mds:
            md.update(*others, **another)
            self.assertTrue(all(md[k] == upd[k] for k in upd.keys()))
            self.assertTrue(all(md[k] == ref[k] for k in ref.keys()
                                if k not in upd))

    def test_10_update_with_a_mdict(self):
        self.check_updated(self.cls(self.up0))

    def test_12_update_with_a_dict(self):
        self.check_updated(dict(self.up0))

    def test_14_update_with_kv_tuples(self):
        self.check_updated(list(self.up0.items()))

    def test_16_update_with_invalid(self):
        md0 = self.cls(self.od0)
        self.assertTrue(isinstance(md0, self.cls))
        try:
            raised = False
            md0.update(1)
        except (ValueError, TypeError):
            raised = True

        self.assertTrue(raised)

    def test_20_update_with_a_odict_and_kwargs(self):
        other = self.up0.copy()
        another = TT.OrderedDict((("b", other["b"]), ))
        del other["b"]

        self.check_updated(other, **another)


class Test_20_UpdateWoReplaceDict(Test_10_UpdateWithReplaceDict):

    cls = TT.UpdateWoReplaceDict

    def check_updated(self, *others, **another):
        upd = self.up0
        ref = self.mds[0].copy()

        for md in self.mds:
            md.update(*others, **another)
            self.assertTrue(all(md[k] == upd[k] for k in upd.keys()
                                if k not in ref))
            self.assertTrue(all(md[k] == ref[k] for k in ref.keys()))


class Test_30_UpdateWithMergeDict(Test_10_UpdateWithReplaceDict):

    cls = TT.UpdateWithMergeDict
    replaced_keys = "a b d".split()

    def check_updated(self, *others, **another):
        upd = self.up0
        ref = self.mds[0].copy()

        for md in self.mds:
            md.update(*others, **another)
            self.assertTrue(all(md[k] == upd[k] for k in self.replaced_keys))
            self.assertTrue(all(md["c"][k] == upd["c"][k] for k
                                in upd["c"].keys()))
            self.assertTrue(all(md[k] == ref[k] for k in ref.keys()
                                if k not in upd))


class Test_32_UpdateWithMergeDict_kept(Test_10_UpdateWithReplaceDict):

    class UWMDK(TT.UpdateWithMergeDict):
        keep = True

    cls = UWMDK

    def check_updated(self, *others, **another):
        upd = self.up0
        ref = self.mds[0].copy()

        for md in self.mds:
            md.update(*others, **another)
            self.assertTrue(all(md[k] == upd[k] for k in ["d"]))
            self.assertEqual(md["c"], ref["c"])
            self.assertTrue(all(md[k] == ref[k] for k in ref.keys()
                                if k not in upd))


class Test_34_UpdateWithMergeListsDict(Test_30_UpdateWithMergeDict):

    cls = TT.UpdateWithMergeListsDict
    replaced_keys = "a d".split()

    def check_updated(self, *others, **another):
        tcls = Test_34_UpdateWithMergeListsDict
        super(tcls, self).check_updated(*others, **another)
        for md in self.mds:
            self.assertEqual(md["b"], [1, 3, 0])


class Test_40_UpdateWithReplaceOrderedDict(Test_10_UpdateWithReplaceDict):

    cls = TT.UpdateWithReplaceOrderedDict

    def setUp(self):
        self.mds = (self.cls(self.od0), self.cls(self.od0.items()))

    def check_updated(self, *others, **another):
        tcls = Test_40_UpdateWithReplaceOrderedDict
        super(tcls, self).check_updated(*others, **another)
        for md in self.mds:
            self.assertEqual(list(md.keys()), "a b c f d".split())


class Test_50_UpdateWoReplaceOrderedDict(Test_20_UpdateWoReplaceDict):

    cls = TT.UpdateWoReplaceOrderedDict

    def setUp(self):
        self.mds = (self.cls(self.od0), self.cls(self.od0.items()))

    def check_updated(self, *others, **another):
        tcls = Test_50_UpdateWoReplaceOrderedDict
        super(tcls, self).check_updated(*others, **another)
        for md in self.mds:
            self.assertEqual(list(md.keys()), "a b c f d".split())


class Test_60_UpdateWithMergeOrderedDict(Test_30_UpdateWithMergeDict):

    cls = TT.UpdateWithMergeOrderedDict

    def setUp(self):
        self.mds = (self.cls(self.od0), self.cls(self.od0.items()))

    def check_updated(self, *others, **another):
        tcls = Test_60_UpdateWithMergeOrderedDict
        super(tcls, self).check_updated(*others, **another)
        for md in self.mds:
            self.assertEqual(list(md.keys()), "a b c f d".split())


_TCLS = Test_34_UpdateWithMergeListsDict


class Test_62_UpdateWithMergeListsOrderedDict(_TCLS):

    cls = TT.UpdateWithMergeListsOrderedDict

    def setUp(self):
        self.mds = (self.cls(self.od0), self.cls(self.od0.items()))

    def check_updated(self, *others, **another):
        tcls = Test_62_UpdateWithMergeListsOrderedDict
        super(tcls, self).check_updated(*others, **another)
        for md in self.mds:
            self.assertEqual(list(md.keys()), "a b c f d".split())

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Red Hat, Inc.
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato redhat.com>
# License: MIT
#
# .. note:: suppress warning about import error of ordereddict only for py-2.6.
# pylint: disable=import-error
"""Some dict-like classes support merge operations.
"""
from __future__ import absolute_import

import m9dicts.globals as MG
import m9dicts.utils

from m9dicts.compat import OrderedDict


def _are_list_like(*objs):
    """
    >>> _are_list_like([], (), [x for x in range(10)], (x for x in range(4)))
    True
    >>> _are_list_like([], {})
    False
    >>> _are_list_like([], "aaa")
    False
    """
    return all(m9dicts.utils.is_list_like(obj) for obj in objs)


class UpdateWithReplaceDict(dict):
    """
    Replace value of self with other's if both have same keys on update.
    Otherwise, just keep the value of self.

    >>> od0 = OrderedDict((("a", 1), ("b", [1, 3]), ("c", "abc"), ("f", None)))
    >>> md0 = UpdateWithReplaceDict(od0)
    >>> upd = UpdateWithReplaceDict(a=2, b=[0, 1], c=dict(d="d", e=1), d="d")
    >>> (ref, md1, md2, md3) = [md0.copy() for _ in range(4)]
    >>> md0.update(upd)
    >>> md1.update(dict(upd))
    >>> md2.update(list(upd.items()))
    >>> md3.update(**upd)
    >>> for md in (md0, md1, md2, md3):
    ...     assert all(md[k] == upd[k] for k in upd.keys())
    ...     assert all(md[k] == ref[k] for k in ref.keys() if k not in upd)
    >>> md5 = UpdateWithReplaceDict(1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...
    """
    def _update(self, other, key, *args):
        """
        :param other: a dict[-like] object or a list of (key, value) tuples
        :param key: object key
        :param args: [] or (value, ...)
        """
        self[key] = args[0] if args else other[key]

    def update(self, *others, **another):
        """
        :param others: a list of dict[-like] objects or (key, value) tuples
        :param another: optional keyword arguments to update self more

        .. seealso:: Document of dict.update
        """
        for other in others:
            if hasattr(other, "keys"):
                for key in other.keys():
                    self._update(other, key)
            else:
                try:
                    for key, val in other:
                        self._update(other, key, val)
                except (ValueError, TypeError) as exc:
                    raise type(exc)(str(exc) + " other=%r" % other)

        for key in another.keys():
            self._update(another, key)


class UpdateWoReplaceDict(UpdateWithReplaceDict):
    """
    Never update (replace) the value of self with other's, that is, only the
    values self does not have its key will be added on update.

    >>> od0 = OrderedDict((("a", 1), ("b", [1, 3]), ("c", "abc"),
    ...                    ("f", None)))
    >>> md0 = UpdateWoReplaceDict(od0)
    >>> ref = md0.copy()
    >>> upd = UpdateWoReplaceDict(a=2, b=[0, 1], c="xyz", d=None)
    >>> md0.update(upd)
    >>> all(md0[k] == upd[k] for k in upd.keys() if k not in ref)
    True
    >>> all(md0[k] == ref[k] for k in ref.keys())
    True
    """
    def _update(self, other, key, *args):
        """
        :param other: a dict[-like] object or a list of (key, value) tuples
        :param key: object key
        :param args: [] or (value, ...)
        """
        if key not in self:
            self[key] = args[0] if args else other[key]


class UpdateWithMergeDict(UpdateWithReplaceDict):
    """
    Merge the value of self with other's recursively. Behavior of merge will be
    vary depends on types of original and new values.

    - dict-like vs. dict-like -> merge recursively
    - list vs. list -> vary depends on `merge_lists`. see its description.
    - other objects vs. any -> vary depends on `keep`. see its description.

    class variables:

        - merge_lists: Merge not only dicts but also lists. For example,

            [1, 2, 3], [3, 4] ==> [1, 2, 3, 4]
            [1, 2, 2], [2, 4] ==> [1, 2, 2, 4]

        - keep: Keep original value if type of original value is not a dict nor
          list. It will be simply replaced with new value by default.

    >>> od0 = OrderedDict((("c", 2), ("d", 3)))
    >>> od1 = OrderedDict((("c", 4), ("d", 5), ("g", None)))
    >>> md0 = UpdateWithMergeDict((("a", 1), ("b", UpdateWithMergeDict(od0)),
    ...                            ("e", [1, 2, 2]), ("f", None)))
    >>> upd = dict(a=2, b=od1, e=[2, 3, 4])
    >>> ref = md0.copy()
    >>> md0.update(upd)
    >>> all(md0[k] == upd[k] for k in ("a", "e"))  # vary depends on 'keep'.
    True
    >>> all(md0["b"][k] == ref["b"][k] for k in ref["b"].keys())
    True
    >>> all(md0[k] == ref[k] for k in ref.keys() if k not in upd)
    True
    >>> class Foo(UpdateWithMergeDict):
    ...     keep = True
    >>> md1 = Foo(a=1, b=2)
    >>> md1.update(a=2, c=3)
    >>> md1["a"]
    1
    >>> md1["c"]
    3
    """
    merge_lists = False
    keep = False

    def _merge_list(self, key, lst):
        """
        :param key: self[key] will be updated
        :param lst: Other list to merge
        """
        self[key] += [x for x in lst if x not in self[key]]

    def _merge_other(self, key, val):
        """
        :param key: self[key] will be updated
        :param val: Other val to merge (update/replace)
        """
        self[key] = val  # Just overwrite it by default implementation.

    def _update(self, other, key, *args):
        """
        :param other: a dict[-like] object or a list of (key, value) tuples
        :param key: object key
        :param args: [] or (value, ...)
        """
        val = args[0] if args else other[key]
        if key in self:
            val0 = self[key]  # Original value
            if m9dicts.utils.is_dict_like(val0):  # It needs recursive updates.
                self[key].update(val)
            elif self.merge_lists and _are_list_like(val, val0):
                self._merge_list(key, val)
            elif not self.keep:
                self._merge_other(key, val)
        else:
            self[key] = val


class UpdateWithMergeListsDict(UpdateWithMergeDict):
    """
    Similar to UpdateWithMergeDict but merge lists by default.

    >>> md0 = UpdateWithMergeListsDict(a=[1, 2, 3])
    >>> md0.update(a=[4, 4, 5])
    >>> md0["a"]
    [1, 2, 3, 4, 4, 5]
    """
    merge_lists = True


class UpdateWithReplaceOrderedDict(UpdateWithReplaceDict, OrderedDict):
    """
    Similar to UpdateWithReplaceDict but keep keys' order like OrderedDict.

    >>> od0 = OrderedDict((("a", 1), ("b", [1, 3]), ("c", "abc"), ("f", None)))
    >>> od1 = OrderedDict((("a", 2), ("b", [0, 1]),
    ...                    ("c", OrderedDict((("d", "d"), ("e", 1)))),
    ...                    ("d", "d")))
    >>> md0 = UpdateWithReplaceOrderedDict(od0)
    >>> md1 = UpdateWithReplaceOrderedDict(od0.items())
    >>> upd = UpdateWithReplaceOrderedDict(od1)
    >>> ref = md0.copy()
    >>> for md in (md0, md1):
    ...     md.update(upd)
    ...     assert all(md0[k] == upd[k] for k in upd.keys())
    ...     assert all(md0[k] == ref[k] for k in ref.keys() if k not in upd)
    ...     assert list(md0.keys()) == ['a', 'b', 'c', 'f', 'd']
    >>>
    """
    pass


class UpdateWoReplaceOrderedDict(UpdateWoReplaceDict, OrderedDict):
    """
    Similar to UpdateWoReplaceDict but keep keys' order like OrderedDict.

    >>> od0 = OrderedDict((("a", 1), ("b", [1, 3]), ("c", "abc"),
    ...                    ("f", None)))
    >>> md0 = UpdateWoReplaceOrderedDict(od0)
    >>> upd = UpdateWoReplaceOrderedDict((("a", 2), ("b", [0, 1]),
    ...                                   ("c", "xyz"), ("d", None)))
    >>> ref = md0.copy()
    >>> md0.update(upd)
    >>> all(md0[k] == upd[k] for k in upd.keys() if k not in ref)
    True
    >>> all(md0[k] == ref[k] for k in ref.keys())
    True
    >>> list(md0.keys())
    ['a', 'b', 'c', 'f', 'd']
    """
    pass


class UpdateWithMergeOrderedDict(UpdateWithMergeDict, OrderedDict):
    """
    Similar to UpdateWithMergeDict but keep keys' order like OrderedDict.

    >>> od0 = OrderedDict((("c", 2), ("d", 3)))
    >>> od1 = OrderedDict((("c", 4), ("d", 5), ("g", None)))
    >>> md0 = UpdateWithMergeOrderedDict(
    ...          OrderedDict((("a", 1),
    ...                       ("b", UpdateWithMergeOrderedDict(od0)),
    ...                       ("e", [1, 2, 2]), ("f", None))))
    >>> ref = md0.copy()
    >>> upd = dict(a=2, b=UpdateWithMergeOrderedDict(od1), e=[2, 3, 4])
    >>> md0.update(upd)
    >>> all(md0[k] == upd[k] for k in ("a", "e"))
    True
    >>> all(md0[k] == ref[k] for k in ref.keys() if k not in upd)
    True
    >>> all(md0["b"][k] == ref["b"][k] for k in ref["b"].keys())
    True
    >>> list(md0.keys())
    ['a', 'b', 'e', 'f']
    >>> list(md0["b"].keys())
    ['c', 'd', 'g']
    """
    pass


class UpdateWithMergeListsOrderedDict(UpdateWithMergeListsDict, OrderedDict):
    """
    Similar to UpdateWithMergeListsDict but keep keys' order like OrderedDict.

    >>> md0 = UpdateWithMergeListsOrderedDict((("a", [1, 2, 3]), ('b', 0)))
    >>> md0.update(a=[4, 4, 5])
    >>> md0["a"]
    [1, 2, 3, 4, 4, 5]
    >>> list(md0.keys())
    ['a', 'b']
    """
    pass


# Mapppings: (merge_strategy, ordered?) : m9dict class
_MDICTS_MAP = {(MG.MS_REPLACE, True): UpdateWithReplaceOrderedDict,
               (MG.MS_REPLACE, False): UpdateWithReplaceDict,
               (MG.MS_NO_REPLACE, True): UpdateWoReplaceOrderedDict,
               (MG.MS_NO_REPLACE, False): UpdateWoReplaceDict,
               (MG.MS_DICTS, True): UpdateWithMergeOrderedDict,
               (MG.MS_DICTS, False): UpdateWithMergeDict,
               (MG.MS_DICTS_AND_LISTS, True): UpdateWithMergeListsOrderedDict,
               (MG.MS_DICTS_AND_LISTS, False): UpdateWithMergeListsDict}


def get_mdict_class(merge=MG.MS_DICTS, ordered=False):
    """
    Select dict-like class based on merge strategy and orderness of keys.

    :param merge: Specify strategy from MERGE_STRATEGIES of how to merge dicts.
    :param ordered:
        The class keeps the order of insertion keys such as
        UpdateWoReplaceOrderedDict will be chosen if True.

    :return: The dict class object
    """
    return _MDICTS_MAP.get((merge, ordered), UpdateWithMergeDict)

# vim:sw=4:ts=4:et:



#
# Author: Satoru SATOH <ssato redhat.com>
# License: MIT
#
"""CLI frontend module for anyconfig.
"""
from __future__ import absolute_import, print_function

import codecs
import locale
import logging
import optparse
import os
import sys

import anyconfig.api as API
import anyconfig.compat
import anyconfig.globals
import anyconfig.mdicts
import anyconfig.parser


_ENCODING = locale.getdefaultlocale()[1]
API.LOGGER.addHandler(logging.StreamHandler())

if anyconfig.compat.IS_PYTHON_3:
    import io

    _ENCODING = _ENCODING.lower()

    # TODO: What should be done for an error, "AttributeError: '_io.StringIO'
    # object has no attribute 'buffer'"?
    try:
        sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding=_ENCODING)
        sys.stderr = io.TextIOWrapper(sys.stderr.buffer, encoding=_ENCODING)
    except AttributeError:
        pass
else:
    sys.stdout = codecs.getwriter(_ENCODING)(sys.stdout)
    sys.stderr = codecs.getwriter(_ENCODING)(sys.stderr)

USAGE = """\
%prog [Options...] CONF_PATH_OR_PATTERN_0 [CONF_PATH_OR_PATTERN_1 ..]

Examples:
  %prog --list  # -> Supported config types: configobj, ini, json, ...
  # Merge and/or convert input config to output config [file]
  %prog -I yaml -O yaml /etc/xyz/conf.d/a.conf
  %prog -I yaml '/etc/xyz/conf.d/*.conf' -o xyz.conf --otype json
  %prog '/etc/xyz/conf.d/*.json' -o xyz.yml \\
    --atype json -A '{"obsoletes": "syscnf", "conflicts": "syscnf-old"}'
  %prog '/etc/xyz/conf.d/*.json' -o xyz.yml \\
    -A obsoletes:syscnf;conflicts:syscnf-old
  %prog /etc/foo.json /etc/foo/conf.d/x.json /etc/foo/conf.d/y.json
  %prog '/etc/foo.d/*.json' -M noreplace
  # Get/set part of input config
  %prog '/etc/foo.d/*.json' --get a.b.c
  %prog '/etc/foo.d/*.json' --set a.b.c=1
  # Validate with JSON schema or generate JSON schema:
  %prog --validate -S foo.conf.schema.yml '/etc/foo.d/*.xml'
  %prog --gen-schema '/etc/foo.d/*.xml' -o foo.conf.schema.yml"""

DEFAULTS = dict(loglevel=1, list=False, output=None, itype=None,
                otype=None, atype=None, merge=API.MS_DICTS,
                ignore_missing=False, template=False, env=False,
                schema=None, validate=False, gen_schema=False)


def to_log_level(level):
    """
    :param level: Logging level in int = 0 .. 2

    >>> to_log_level(0) == logging.WARN
    True
    >>> to_log_level(5)  # doctest: +IGNORE_EXCEPTION_DETAIL, +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: wrong log level passed: 5
    >>>
    """
    if not (level >= 0 and level < 3):
        raise ValueError("wrong log level passed: " + str(level))

    return [logging.WARN, logging.INFO, logging.DEBUG][level]


_ATYPE_HELP_FMT = """\
Explicitly select type of argument to provide configs from %s.

If this option is not set, original parser is used: 'K:V' will become {K: V},
'K:V_0,V_1,..' will become {K: [V_0, V_1, ...]}, and 'K_0:V_0;K_1:V_1' will
become {K_0: V_0, K_1: V_1} (where the tyep of K is str, type of V is one of
Int, str, etc."""

_GET_HELP = ("Specify key path to get part of config, for example, "
             "'--get a.b.c' to config {'a': {'b': {'c': 0, 'd': 1}}} "
             "gives 0 and '--get a.b' to the same config gives "
             "{'c': 0, 'd': 1}. Path expression can be JSON Pointer "
             "expression (http://tools.ietf.org/html/rfc6901) such like "
             "'', '/a~1b', '/m~0n'.")
_SET_HELP = ("Specify key path to set (update) part of config, for "
             "example, '--set a.b.c=1' to a config {'a': {'b': {'c': 0, "
             "'d': 1}}} gives {'a': {'b': {'c': 1, 'd': 1}}}.")


def parse_args(argv=None, defaults=None):
    """
    Make up an option and arguments parser.

    :param defaults: Default option values
    """
    if defaults is None:
        defaults = DEFAULTS

    ctypes = API.list_types()
    ctypes_s = ", ".join(ctypes)
    type_help = "Select type of %s config files from " + \
        ctypes_s + " [Automatically detected by file ext]"

    mts = API.MERGE_STRATEGIES
    mts_s = ", ".join(mts)
    mt_help = "Select strategy to merge multiple configs from " + \
        mts_s + " [%(merge)s]" % defaults

    parser = optparse.OptionParser(USAGE, version="%%prog %s" %
                                   anyconfig.globals.VERSION)
    parser.set_defaults(**defaults)

    lpog = optparse.OptionGroup(parser, "List specific options")
    lpog.add_option("-L", "--list", help="List supported config types",
                    action="store_true")
    parser.add_option_group(lpog)

    spog = optparse.OptionGroup(parser, "Schema specific options")
    spog.add_option("", "--validate", action="store_true",
                    help="Only validate input files and do not output. "
                         "You must specify schema file with -S/--schema "
                         "option.")
    spog.add_option("", "--gen-schema", action="store_true",
                    help="Generate JSON schema for givne config file[s] "
                         "and output it instead of (merged) configuration.")
    parser.add_option_group(spog)

    gspog = optparse.OptionGroup(parser, "Get/set options")
    gspog.add_option("", "--get", help=_GET_HELP)
    gspog.add_option("", "--set", help=_SET_HELP)
    parser.add_option_group(gspog)

    parser.add_option("-o", "--output", help="Output file path")
    parser.add_option("-I", "--itype", choices=ctypes,
                      help=(type_help % "Input"))
    parser.add_option("-O", "--otype", choices=ctypes,
                      help=(type_help % "Output"))
    parser.add_option("-M", "--merge", choices=mts, help=mt_help)
    parser.add_option("-A", "--args", help="Argument configs to override")
    parser.add_option("", "--atype", choices=ctypes,
                      help=_ATYPE_HELP_FMT % ctypes_s)

    parser.add_option("-x", "--ignore-missing", action="store_true",
                      help="Ignore missing input files")
    parser.add_option("-T", "--template", action="store_true",
                      help="Enable template config support")
    parser.add_option("-E", "--env", action="store_true",
                      help="Load configuration defaults from "
                           "environment values")
    parser.add_option("-S", "--schema", help="Specify Schema file[s] path")
    parser.add_option("-s", "--silent", action="store_const", dest="loglevel",
                      const=0, help="Silent or quiet mode")
    parser.add_option("-q", "--quiet", action="store_const", dest="loglevel",
                      const=0, help="Same as --silent option")
    parser.add_option("-v", "--verbose", action="store_const", dest="loglevel",
                      const=2, help="Verbose mode")

    if argv is None:
        argv = sys.argv

    (options, args) = parser.parse_args(argv[1:])
    return (parser, options, args)


def _exit_with_output(content, exit_code=0):
    """
    Exit the program with printing out messages.

    :param content: content to print out
    :param exit_code: Exit code
    """
    (sys.stdout if exit_code == 0 else sys.stderr).write(content + "\n")
    sys.exit(exit_code)


def _check_options_and_args(parser, options, args):
    """
    Show supported config format types or usage.

    :param parser: Option parser object
    :param options: Options optparse.OptionParser.parse_args returns
    :param args: Arguments optparse.OptionParser.parse_args returns
    """
    if not args:
        if options.list:
            tlist = ", ".join(API.list_types())
            _exit_with_output("Supported config types: " + tlist)
        else:
            parser.print_usage()
            sys.exit(1)

    if options.validate and options.schema is None:
        _exit_with_output("--validate option requires --scheme option", 1)


def _exit_if_load_failure(cnf, msg):
    """
    :param cnf: Loaded configuration object or None indicates load failure
    :param msg: Message to print out if failure
    """
    if cnf is None:
        _exit_with_output(msg, 1)


def _exit_if_only_to_validate(only_to_validate):
    """
    :param only_to_validate: True if it's only to validate
    """
    if only_to_validate:
        _exit_with_output("Validation succeds")


def _do_get(cnf, get_path):
    """
    :param cnf: Configuration object to print out
    :param get_path: key path given in --get option
    :return: updated Configuration object if no error
    """
    (cnf, err) = API.get(cnf, get_path)
    if cnf is None:  # Failed to get the result.
        _exit_with_output("Failed to get result: err=%s" % err, 1)

    return cnf


def _output_result(cnf, outpath, otype, inpath, itype):
    """
    :param cnf: Configuration object to print out
    :param outpath: Output file path or None
    :param otype: Output type or None
    :param inpath: Input file path
    :param itype: Input type or None
    """
    if not outpath or outpath == "-":
        outpath = sys.stdout
        if otype is None:
            if itype is None:
                try:
                    otype = API.find_loader(inpath).type()
                except AttributeError:
                    _exit_with_output("Specify inpath and/or outpath type[s] "
                                      "with -I/--itype or -O/--otype option "
                                      "explicitly", 1)
            else:
                otype = itype

    if anyconfig.mdicts.is_dict_like(cnf):
        API.dump(cnf, outpath, otype)
    else:
        _exit_with_output(str(cnf))  # Output primitive types as it is.


def main(argv=None):
    """
    :param argv: Argument list to parse or None (sys.argv will be set).
    """
    (parser, options, args) = parse_args(argv=argv)
    API.LOGGER.setLevel(to_log_level(options.loglevel))

    _check_options_and_args(parser, options, args)

    cnf = API.to_container(os.environ.copy() if options.env else {})
    diff = API.load(args, options.itype,
                    ignore_missing=options.ignore_missing,
                    ac_merge=options.merge, ac_template=options.template,
                    ac_schema=options.schema)

    _exit_if_load_failure(diff, "Failed to load: args=%s" % ", ".join(args))
    cnf.update(diff)

    if options.args:
        diff = anyconfig.parser.parse(options.args)
        cnf.update(diff)

    _exit_if_only_to_validate(options.validate)

    if options.gen_schema:
        cnf = API.gen_schema(cnf)

    if options.get:
        cnf = _do_get(cnf, options.get)

    if options.set:
        (key, val) = options.set.split('=')
        API.set_(cnf, key, anyconfig.parser.parse(val))

    _output_result(cnf, options.output, options.otype, args[0], options.itype)


if __name__ == '__main__':
    main(sys.argv)

# vim:sw=4:ts=4:et:



"""
.. module:: m9dicts
   :synopsis: dict-like objects support recursive merge operations
"""
from .globals import (
    MS_REPLACE, MS_NO_REPLACE, MS_DICTS, MS_DICTS_AND_LISTS, MERGE_STRATEGIES,
    NTPL_CLS_KEY
)  # flake8: noqa
from .dicts import (
    UpdateWithReplaceDict, UpdateWoReplaceDict, UpdateWithMergeDict,
    UpdateWithMergeListsDict, UpdateWithReplaceOrderedDict,
    UpdateWoReplaceOrderedDict, UpdateWithMergeOrderedDict,
    UpdateWithMergeListsOrderedDict
)  # flake8: noqa
from .api import get, set_, make, convert_to  # flake8: noqa
from .utils import is_dict_like, is_namedtuple, is_list_like

__author__ = "Satoru SATOH <ssato@redhat.com>"
__version__ = "0.2.0"

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2012 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=missing-docstring, invalid-name
from __future__ import absolute_import

import copy
import os.path
import unittest

import anyconfig.backend.xml as TT
import anyconfig.backend.tests.ini
import anyconfig.tests.common

from anyconfig.tests.common import dicts_equal, to_bytes
from anyconfig.mdicts import to_container


CNF_0_S = """<?xml version="1.0" encoding="UTF-8"?>
<config name='foo'>
  <a>0</a>
  <b id="b0">bbb</b>
  <sect0>
    <c>x, y, z</c>
  </sect0>
</config>
"""

CNF_0 = {'config': {'@attrs': {'name': 'foo'},
                    '@children': [{'a': {'@text': '0'}},
                                  {'b': {'@attrs': {'id': 'b0'},
                                         '@text': 'bbb'}},
                                  {'sect0': {
                                      '@children': [{'c': {
                                          '@text': 'x, y, z'
                                      }}]}}]}}


class Test00(unittest.TestCase):

    def setUp(self):
        self.root = TT.ET.Element("config", attrib=dict(name="foo"))
        celm0 = TT.ET.SubElement(self.root, "a")
        celm0.text = '0'
        celm1 = TT.ET.SubElement(self.root, "b", attrib=dict(id="b0", ))
        celm1.text = "bbb"
        self.tree = TT.ET.ElementTree(self.root)

        self.cnf = copy.deepcopy(CNF_0)
        del self.cnf["config"]["@children"][-1]

        self.cnf_s = to_bytes('<config name="foo"><a>0</a>'
                              '<b id="b0">bbb</b></config>')

    def test_10_etree_to_container__empty(self):
        self.assertEqual(TT.etree_to_container(None, dict), {})

    def test_12_etree_to_container(self):
        cnf = TT.etree_to_container(self.root, to_container)
        self.assertTrue(dicts_equal(cnf, self.cnf), str(cnf))

    def test_20_container_to_etree__empty(self):
        # This should not happen but just in case.
        self.assertTrue(TT.container_to_etree("aaa", dict) is None)

    def test_22_container_to_etree(self):
        tree = TT.container_to_etree(self.cnf, to_container)
        buf = TT.BytesIO()
        tree.write(buf)
        cnf_s = buf.getvalue()
        self.assertEqual(cnf_s, self.cnf_s, cnf_s)


class Test10(anyconfig.backend.tests.ini.Test10):

    cnf = CNF_0
    cnf_s = CNF_0_S
    load_options = dump_options = dict(dummy="this will be ignored")

    def setUp(self):
        self.psr = TT.Parser()


class Test20(anyconfig.backend.tests.ini.Test20):

    psr_cls = TT.Parser
    cnf = CNF_0
    cnf_s = CNF_0_S
    cnf_fn = "conf0.xml"

    def setUp(self):
        self.psr = TT.Parser()
        self.workdir = anyconfig.tests.common.setup_workdir()
        self.cpath = os.path.join(self.workdir, self.cnf_fn)
        open(self.cpath, 'w').write(self.cnf_s)


# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Red Hat, Inc.
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato redhat.com>
# License: MIT
#
"""Functions operate on m9dicts objects.

.. versionchanged: 0.1.0

   - splitted / forked from python-anyconfig; old history was available in its
     mdict branch:
     https://github.com/ssato/python-anyconfig/blob/mdict/anyconfig/mergeabledict.py

"""
from __future__ import absolute_import

import collections
import functools
import operator
import re

import m9dicts.compat
import m9dicts.globals
import m9dicts.dicts
import m9dicts.utils


PATH_SEPS = ('/', '.')

_JSNP_GET_ARRAY_IDX_REG = re.compile(r"(?:0|[1-9][0-9]*)")
_JSNP_SET_ARRAY_IDX = re.compile(r"(?:0|[1-9][0-9]*|-)")


def _jsnp_unescape(jsn_s):
    """
    Parse and decode given encoded JSON Pointer expression, convert ~1 to
    / and ~0 to ~.

    .. note:: JSON Pointer: http://tools.ietf.org/html/rfc6901

    >>> _jsnp_unescape("/a~1b")
    '/a/b'
    >>> _jsnp_unescape("~1aaa~1~0bbb")
    '/aaa/~bbb'
    """
    return jsn_s.replace('~1', '/').replace('~0', '~')


def _split_path(path, seps=PATH_SEPS):
    """
    Parse path expression and return list of path items.

    :param path: Path expression may contain separator chars.
    :param seps: Separator char candidates.
    :return: A list of keys to fetch object[s] later.

    >>> assert _split_path('') == []
    >>> assert _split_path('/') == ['']  # JSON Pointer spec expects this.
    >>> for p in ('/a', '.a', 'a', 'a.'):
    ...     assert _split_path(p) == ['a'], p
    >>> assert _split_path('/a/b/c') == _split_path('a.b.c') == ['a', 'b', 'c']
    >>> assert _split_path('abc') == ['abc']
    """
    if not path:
        return []

    for sep in seps:
        if sep in path:
            if path == sep:  # Special case, '/' or '.' only.
                return ['']
            return [x for x in path.split(sep) if x]

    return [path]


def mk_nested_dic(path, val, seps=PATH_SEPS):
    """
    Make a nested dict iteratively.

    :param path: Path expression to make a nested dict
    :param val: Value to set
    :param seps: Separator char candidates

    >>> mk_nested_dic("a.b.c", 1)
    {'a': {'b': {'c': 1}}}
    >>> mk_nested_dic("/a/b/c", 1)
    {'a': {'b': {'c': 1}}}
    """
    ret = None
    for key in reversed(_split_path(path, seps)):
        ret = {key: val if ret is None else ret.copy()}

    return ret


def get(dic, path, seps=PATH_SEPS, idx_reg=_JSNP_GET_ARRAY_IDX_REG):
    """getter for nested dicts.

    :param dic: a dict[-like] object
    :param path: Path expression to point object wanted
    :param seps: Separator char candidates
    :return: A tuple of (result_object, error_message)

    >>> d = {'a': {'b': {'c': 0, 'd': [1, 2]}}, '': 3}
    >>> assert get(d, '/') == (3, '')  # key becomes '' (empty string).
    >>> assert get(d, "/a/b/c") == (0, '')
    >>> sorted(get(d, "a.b")[0].items())
    [('c', 0), ('d', [1, 2])]
    >>> (get(d, "a.b.d"), get(d, "/a/b/d/1"))
    (([1, 2], ''), (2, ''))
    >>> get(d, "a.b.key_not_exist")  # doctest: +ELLIPSIS
    (None, "'...'")
    >>> get(d, "/a/b/d/2")
    (None, 'list index out of range')
    >>> get(d, "/a/b/d/-")  # doctest: +ELLIPSIS
    (None, 'list indices must be integers...')
    """
    items = [_jsnp_unescape(p) for p in _split_path(path, seps)]
    if not items:
        return (dic, '')
    try:
        if len(items) == 1:
            return (dic[items[0]], '')

        prnt = functools.reduce(operator.getitem, items[:-1], dic)
        arr = m9dicts.utils.is_list_like(prnt) and idx_reg.match(items[-1])
        return (prnt[int(items[-1])], '') if arr else (prnt[items[-1]], '')

    except (TypeError, KeyError, IndexError) as exc:
        return (None, str(exc))


def set_(dic, path, val, seps=PATH_SEPS):
    """setter for nested dicts.

    :param dic: a dict[-like] object support recursive merge operations
    :param path: Path expression to point object wanted
    :param seps: Separator char candidates

    >>> d = dict(a=1, b=dict(c=2, ))
    >>> set_(d, 'a.b.d', 3)
    >>> d['a']['b']['d']
    3
    """
    dic.update(mk_nested_dic(path, val, seps))


def check_merge(merge):
    """Check if given `merge` is valid and ValueError will be raised if not.
    """
    if merge not in m9dicts.globals.MERGE_STRATEGIES:
        raise ValueError("Wrong merge strategy: %r" % merge)


def _make_from_namedtuple(obj, merge=m9dicts.globals.MS_DICTS,
                          _ntpl_cls_key=m9dicts.globals.NTPL_CLS_KEY,
                          **options):
    """
    :param obj: A namedtuple object
    :param merge:
        Specify strategy from MERGE_STRATEGIES of how to merge results loaded
        from multiple configuration files.
    :param _ntpl_cls_key:
        Special keyword to embedded the class name of namedtuple object to the
        MergeableDict object created. It's a hack and not elegant but I don't
        think there are another ways to make same namedtuple object from the
        MergeableDict object created from it.
    """
    ocls = m9dicts.dicts.get_mdict_class(merge=merge, ordered=True)
    mdict = ocls((k, make(getattr(obj, k), **options)) for k in obj._fields)
    mdict[_ntpl_cls_key] = obj.__class__.__name__

    return mdict


def _make_recur(obj, cls, make_fn, **options):
    """
    :param obj: An original mapping object
    :param cls: Another mapping class to make/convert to
    :param make_fn: Function to make/convert to
    """
    return cls((k, None if v is None else make_fn(v, **options))
               for k, v in obj.items())


def _make_iter(obj, make_fn, **options):
    """
    :param obj: An original mapping object
    :param make_fn: Function to make/convert to
    """
    return type(obj)(make_fn(v, **options) for v in obj)


def make(obj=None, ordered=False, merge=m9dicts.globals.MS_DICTS, **options):
    """
    Factory function to create a dict-like object[s] supports merge operation
    from a mapping or a list of mapping objects such as dict, [dict],
    namedtuple, [namedtuple].

    :param obj: A dict or other object[s] or None
    :param ordered:
        Choose the class keeps key order if True or `obj` is a namedtuple.
    :param merge: see :func:`_make_from_namedtuple` (above).
    :return: A dict-like object[s] supports merge operation or `obj` itself
    """
    check_merge(merge)
    cls = m9dicts.dicts.get_mdict_class(merge=merge, ordered=ordered)
    if obj is None:
        return cls()

    options.update(ordered=ordered, merge=merge)
    if m9dicts.utils.is_dict_like(obj):
        return _make_recur(obj, cls, make, **options)
    elif m9dicts.utils.is_namedtuple(obj):
        return _make_from_namedtuple(obj, **options)
    elif m9dicts.utils.is_list_like(obj):
        return _make_iter(obj, make, **options)
    else:
        return obj


def _convert_to_namedtuple(obj, _ntpl_cls_key=m9dicts.globals.NTPL_CLS_KEY,
                           **options):
    """Convert a dict-like object to a namedtuple.

    :param obj: A m9dicts objects or other primitive object
    :param _ntpl_cls_key: see :func:`_make_from_namedtuple`
    """
    _name = obj.get(_ntpl_cls_key, "NamedTuple")
    _keys = [k for k in obj.keys() if k != _ntpl_cls_key]
    _vals = [convert_to(obj[k], **options) for k in _keys]
    return collections.namedtuple(_name, _keys)(*_vals)


def convert_to(obj, ordered=False, to_namedtuple=False, **options):
    """
    Convert a dict-like object[s] support merge operation to a dict or
    namedtuple object recursively. Borrowed basic idea and implementation from
    bunch.unbunchify. (bunch is distributed under MIT license same as this.)

    .. note::
       - Given `obj` doesn't keep key order and if `to_namedtuple` is True,
         then the order of fields of result namedtuple object becomes random.
       - namedtuple object cannot have fields start with '_', So it'll fail if
         to convert dicts has such keys.

    :param obj: A m9dicts objects or other primitive object
    :param ordered: Create an OrderedDict instead of dict to keep the key order
    :param to_namedtuple: Convert `obj` to namedtuple instead of a dict
    :param options:
        Optional keyword arguments such as _ntpl_cls_key. see
        :func:`_make_from_namedtuple` for more its details.

    :return: A dict or namedtuple object if to_namedtuple is True
    """
    options.update(ordered=ordered, to_namedtuple=to_namedtuple)
    if m9dicts.utils.is_dict_like(obj):
        if to_namedtuple:
            return _convert_to_namedtuple(obj, **options)
        else:
            cls = m9dicts.compat.OrderedDict if ordered else dict
            return _make_recur(obj, cls, convert_to, **options)
    elif m9dicts.utils.is_list_like(obj):
        return _make_iter(obj, convert_to, **options)
    else:
        return obj

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato at redhat.com>
#
# pylint: disable=missing-docstring
from __future__ import absolute_import
import unittest


def dicts_equal(lhs, rhs):
    """
    Check dicts' equality.
    """
    if len(lhs.keys()) != len(rhs.keys()):
        return False

    for key, val in rhs.items():
        val_ref = lhs.get(key, None)
        if val != val_ref:
            return False

    return True


class Test00(unittest.TestCase):

    def test_10_dicts_equal(self):
        self.assertTrue(dicts_equal({}, {}))
        self.assertFalse(dicts_equal({}, {'a': 1}))

        dic0 = {'a': 1}
        dic1 = {'a': [1, 2, 3]}
        self.assertTrue(dicts_equal(dic0, dic0))
        self.assertFalse(dicts_equal(dic0, dic1))

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# Suppress:
# - false-positive warn at '... pkg_resources ...' line
# - import positions after some globals are defined
# pylint: disable=no-member,wrong-import-position
"""A module to aggregate config parser (loader/dumper) backends.
"""
from __future__ import absolute_import

import itertools
import logging
import operator
import pkg_resources

import anyconfig.compat
import anyconfig.utils

import anyconfig.backend.base
import anyconfig.backend.ini
import anyconfig.backend.json
import anyconfig.backend.properties
import anyconfig.backend.shellvars
import anyconfig.backend.xml

LOGGER = logging.getLogger(__name__)
PARSERS = [anyconfig.backend.ini.Parser, anyconfig.backend.json.Parser,
           anyconfig.backend.properties.Parser,
           anyconfig.backend.shellvars.Parser, anyconfig.backend.xml.Parser]

_NA_MSG = "%s is not available. Disabled %s support."

try:
    import anyconfig.backend.yaml
    PARSERS.append(anyconfig.backend.yaml.Parser)
except ImportError:
    LOGGER.info(_NA_MSG, "yaml module", "YAML")

try:
    import anyconfig.backend.configobj
    PARSERS.append(anyconfig.backend.configobj.Parser)
except ImportError:
    LOGGER.info(_NA_MSG, "ConfigObj module", "its")

try:
    import anyconfig.backend.msgpack
    PARSERS.append(anyconfig.backend.msgpack.Parser)
except ImportError:
    LOGGER.info(_NA_MSG, "msgpack module", "MessagePack")

try:
    import anyconfig.backend.toml
    PARSERS.append(anyconfig.backend.toml.Parser)
except ImportError:
    LOGGER.info(_NA_MSG, "toml module", "TOML")

try:
    import anyconfig.backend.bson
    PARSERS.append(anyconfig.backend.bson.Parser)
except ImportError:
    LOGGER.info(_NA_MSG, "bson module in pymongo package", "BSON")

for e in pkg_resources.iter_entry_points("anyconfig_backends"):
    try:
        PARSERS.append(e.load())
    except ImportError:
        continue


def fst(tpl):
    """
    >>> fst((0, 1))
    0
    """
    return tpl[0]


def snd(tpl):
    """
    >>> snd((0, 1))
    1
    """
    return tpl[1]


def groupby_key(itr, keyfunc):
    """
    An wrapper function around itertools.groupby

    :param itr: Iterable object, a list/tuple/genrator, etc.
    :param keyfunc: Key function to sort `itr`.

    >>> itr = [("a", 1), ("b", -1), ("c", 1)]
    >>> res = groupby_key(itr, operator.itemgetter(1))
    >>> [(key, tuple(grp)) for key, grp in res]
    [(-1, (('b', -1),)), (1, (('a', 1), ('c', 1)))]
    """
    return itertools.groupby(sorted(itr, key=keyfunc), key=keyfunc)


def uniq(iterable, **kwopts):
    """sorted + uniq

    .. note::
       sorted(set(iterable), key=iterable.index) cannot be used for any
       iterables (generator, a list of dicts, etc.), I guess.

    :param iterable: Iterable objects, a list, generator, iterator, etc.
    :param kwopts: Keyword options passed to sorted()
    :return: a sorted list of items in iterable

    >>> uniq([1, 2, 3, 1, 2])
    [1, 2, 3]
    >>> uniq((i for i in (2, 10, 3, 2, 5, 1, 7, 3)))
    [1, 2, 3, 5, 7, 10]
    >>> uniq(({str(i): i} for i in (2, 10, 3, 2, 5, 1, 7, 3)),
    ...      key=lambda d: int(list(d.keys())[0]))
    [{'1': 1}, {'2': 2}, {'3': 3}, {'5': 5}, {'7': 7}, {'10': 10}]
    """
    return [t[0] for t in itertools.groupby(sorted(iterable, **kwopts))]


def is_parser(obj):
    """
    :return: True if given `obj` is an instance of parser.

    >>> is_parser("ini")
    False
    >>> is_parser(anyconfig.backend.base.Parser)
    False
    >>> is_parser(anyconfig.backend.base.Parser())
    True
    """
    return isinstance(obj, anyconfig.backend.base.Parser)


def list_parsers_by_type(cps=None):
    """
    :return: List (generator) of (config_type, [config_parser])
    """
    if cps is None:
        cps = PARSERS

    return ((t, sorted(p, key=operator.methodcaller("priority"))) for t, p
            in groupby_key(cps, operator.methodcaller("type")))


def _list_xppairs(xps):
    """List config parsers by priority.
    """
    return sorted((snd(xp) for xp in xps),
                  key=operator.methodcaller("priority"))


def list_parsers_by_extension(cps=None):
    """
    :return: List (generator) of (config_ext, [config_parser])
    """
    if cps is None:
        cps = PARSERS

    cps_by_ext = anyconfig.utils.concat(([(x, p) for x in p.extensions()] for p
                                         in cps))

    return ((x, _list_xppairs(xps)) for x, xps in groupby_key(cps_by_ext, fst))


def find_by_file(path_or_stream, cps=None, is_path_=False):
    """
    Find config parser by the extension of file `path_or_stream`, file path or
    stream (a file or file-like objects).

    :param path_or_stream: Config file path or file/file-like object
    :param cps: A list of pairs :: (type, parser_class)
    :param is_path_: True if given `path_or_stream` is a file path

    :return: Config Parser class found

    >>> find_by_file("a.missing_cnf_ext") is None
    True
    >>> strm = anyconfig.compat.StringIO()
    >>> find_by_file(strm) is None
    True
    >>> find_by_file("a.json")
    <class 'anyconfig.backend.json.Parser'>
    >>> find_by_file("a.json", is_path_=True)
    <class 'anyconfig.backend.json.Parser'>
    """
    if cps is None:
        cps = PARSERS

    if not is_path_ and not anyconfig.utils.is_path(path_or_stream):
        path_or_stream = anyconfig.utils.get_path_from_stream(path_or_stream)
        if path_or_stream is None:
            return None  # There is no way to detect file path.

    ext_ref = anyconfig.utils.get_file_extension(path_or_stream)
    for ext, psrs in list_parsers_by_extension(cps):
        if ext == ext_ref:
            return psrs[-1]

    return None


def find_by_type(cptype, cps=None):
    """
    Find config parser by file's extension.

    :param cptype: Config file's type
    :param cps: A list of pairs :: (type, parser_class)
    :return: Config Parser class found

    >>> find_by_type("missing_type") is None
    True
    """
    if cps is None:
        cps = PARSERS

    for type_, psrs in list_parsers_by_type(cps):
        if type_ == cptype:
            return psrs[-1] or None

    return None


def find_parser(path_or_stream, forced_type=None, is_path_=False):
    """
    Find out config parser object appropriate to load from a file of given path
    or file/file-like object.

    :param path_or_stream: Configuration file path or file / file-like object
    :param forced_type: Forced configuration parser type
    :param is_path_: True if given `path_or_stream` is a file path

    :return: A tuple of (Parser class or None, "" or error message)

    >>> find_parser(None)
    Traceback (most recent call last):
    ValueError: path_or_stream or forced_type must be some value

    >>> find_parser(None, "ini")
    (<class 'anyconfig.backend.ini.Parser'>, '')
    >>> find_parser(None, "type_not_exist")
    (None, 'No parser found for given type: type_not_exist')

    >>> find_parser("cnf.json")
    (<class 'anyconfig.backend.json.Parser'>, '')
    >>> find_parser("cnf.json", is_path_=True)
    (<class 'anyconfig.backend.json.Parser'>, '')
    >>> find_parser("cnf.ext_not_found")
    (None, 'No parser found for given file: cnf.ext_not_found')
    """
    if not path_or_stream and forced_type is None:
        raise ValueError("path_or_stream or forced_type must be some value")

    err = ""
    if forced_type is not None:
        parser = find_by_type(forced_type)
        if parser is None:
            err = "No parser found for given type: %s" % forced_type
    else:
        parser = find_by_file(path_or_stream, is_path_=is_path_)
        if parser is None:
            err = "No parser found for given file: %s" % path_or_stream

    return (parser, err)


def list_types(cps=None):
    """List available config types.
    """
    if cps is None:
        cps = PARSERS

    return uniq(t for t, ps in list_parsers_by_type(cps))

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2016 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
"""
Parser for simple Shell vars' definitions.

.. versionadded:: 0.7.0
   Added an experimental parser for simple shelll vars' definitions w/o shell
   variable expansions nor complex shell statements like conditionals.

- Format to support: Simple shell variables' definitions w/o any shell variable
  expansions nor complex shell statements such as conditionals, etc.
- Requirements: None (built-in)
- Limitations: Currently, it only supports a varialbe defined in a line.
- Special options: None
"""
from __future__ import absolute_import

import logging
import itertools
import re

import anyconfig.backend.base


LOGGER = logging.getLogger(__name__)


def _parseline(line):
    """
    Parse a line contains shell variable definition.

    :param line: A string to parse, must not start with '#' (comment)
    :return: A tuple of (key, value), both key and value may be None

    >>> _parseline("aaa=")
    ('aaa', '')
    >>> _parseline("aaa=bbb")
    ('aaa', 'bbb')
    >>> _parseline("aaa='bb b'")
    ('aaa', 'bb b')
    >>> _parseline('aaa="bb#b"')
    ('aaa', 'bb#b')
    >>> _parseline('aaa="bb\\"b"')
    ('aaa', 'bb"b')
    >>> _parseline("aaa=bbb   # ccc")
    ('aaa', 'bbb')
    """
    match = re.match(r"^\s*(\S+)=(?:(?:"
                     r"(?:\"(.*[^\\])\")|(?:'(.*[^\\])')|"
                     r"(?:([^\"'#\s]+)))?)\s*#*", line)
    if not match:
        LOGGER.warning("Invalid line found: %s", line)
        return (None, None)

    tpl = match.groups()
    vals = list(itertools.dropwhile(lambda x: x is None, tpl[1:]))
    return (tpl[0], vals[0] if vals else '')


def load(stream, to_container=dict):
    """
    Load and parse a file or file-like object `stream` provides simple shell
    variables' definitions.

    :param stream: A file or file like object
    :param to_container:
        Factory function to create a dict-like object to store properties
    :return: Dict-like object holding shell variables' definitions

    >>> from anyconfig.compat import StringIO as to_strm
    >>> load(to_strm(''))
    {}
    >>> load(to_strm("# "))
    {}
    >>> load(to_strm("aaa="))
    {'aaa': ''}
    >>> load(to_strm("aaa=bbb"))
    {'aaa': 'bbb'}
    >>> load(to_strm("aaa=bbb # ..."))
    {'aaa': 'bbb'}
    """
    ret = to_container()

    for line in stream.readlines():
        line = line.rstrip()
        if line is None or not line:
            continue

        (key, val) = _parseline(line)
        if key is None:
            LOGGER.warning("Empty val in the line: %s", line)
            continue

        ret[key] = val

    return ret


class Parser(anyconfig.backend.base.FromStreamLoader,
             anyconfig.backend.base.ToStreamDumper):
    """
    Parser for Java properties files.
    """
    _type = "shellvars"

    def load_from_stream(self, stream, to_container, **kwargs):
        """
        Load config from given file like object `stream`.

        :param stream: A file or file like object of Java properties files
        :param to_container: callble to make a container object
        :param kwargs: optional keyword parameters (ignored)

        :return: Dict-like object holding config parameters
        """
        return load(stream, to_container=to_container)

    def dump_to_stream(self, cnf, stream, **kwargs):
        """
        Dump config `cnf` to a file or file-like object `stream`.

        :param cnf: Java properties config data to dump
        :param stream: Java properties file or file like object
        :param kwargs: backend-specific optional keyword parameters :: dict
        """
        for key, val in anyconfig.compat.iteritems(cnf):
            stream.write("%s='%s'\n" % (key, val))

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2016 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=missing-docstring
from __future__ import absolute_import
import anyconfig.backend.shellvars as TT
import anyconfig.backend.tests.ini

from anyconfig.compat import OrderedDict as ODict


CNF_S = """
a=0
b='bbb'   # a comment
c="ccc"   # an another comment
"""
CNF = ODict((("a", "0"), ("b", "bbb"), ("c", "ccc")))


class Test10(anyconfig.backend.tests.ini.Test10):

    cnf = CNF
    cnf_s = CNF_S

    def setUp(self):
        self.psr = TT.Parser()


class Test20(anyconfig.backend.tests.ini.Test20):

    psr_cls = TT.Parser
    cnf = CNF
    cnf_s = CNF_S
    cnf_fn = "conf.sh"

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Red Hat, Inc.
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato redhat.com>
# License: MIT
#
# .. note:: suppress warning about import error of ordereddict only for py-2.6.
# pylint: disable=import-error
"""Some dict-like classes support merge operations.
"""
from __future__ import absolute_import


# Merge strategies:
MS_REPLACE = "replace"
MS_NO_REPLACE = "noreplace"
MS_DICTS = "merge_dicts"
MS_DICTS_AND_LISTS = "merge_dicts_and_lists"
MERGE_STRATEGIES = (MS_REPLACE, MS_NO_REPLACE, MS_DICTS, MS_DICTS_AND_LISTS)

# .. note::
#    Special keyword to embedded the class name of namedtuple object to the
#    m9dict object created. It's a hack and not elegant but I don't think there
#    are another ways to make # same namedtuple object from the m9dict object
#    created from it.
NTPL_CLS_KEY = "namedtuple_cls__"

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato @ redhat.com>
#
# pylint: disable=missing-docstring,invalid-name
from __future__ import absolute_import

import collections
import unittest
import m9dicts.api as TT
import m9dicts.dicts as MD
import m9dicts.globals as MG

from m9dicts.tests.common import dicts_equal
from m9dicts.compat import OrderedDict


class Test_10_get(unittest.TestCase):

    def test_10_empty_path(self):
        dic = dict(a=1, b=[1, 2])
        (dic2, err) = TT.get(dic, '')
        self.assertEqual(err, '')
        self.assertTrue(dicts_equal(dic2, dic))

    def test_20_json_pointer(self):
        # test case in rfc, http://tools.ietf.org/html/rfc6901
        dic = {"foo": ["bar", "baz"],
               "": 0,
               "a/b": 1,
               "c%d": 2,
               "e^f": 3,
               "g|h": 4,
               r"i\\j": 5,
               r'k\"l': 6,
               " ": 7,
               "m~n": 8}

        self.assertTrue(dicts_equal(TT.get(dic, "")[0], dic))
        self.assertEqual(TT.get(dic, "/foo")[0], ["bar", "baz"])
        self.assertEqual(TT.get(dic, "/foo/0")[0], "bar")
        self.assertEqual(TT.get(dic, "/")[0], 0)
        self.assertEqual(TT.get(dic, "/a~1b")[0], 1)
        self.assertEqual(TT.get(dic, "/c%d")[0], 2)
        self.assertEqual(TT.get(dic, "/e^f")[0], 3)
        self.assertEqual(TT.get(dic, "/g|h")[0], 4)
        self.assertEqual(TT.get(dic, r"/i\\j")[0], 5)
        self.assertEqual(TT.get(dic, r'/k\"l')[0], 6)
        self.assertEqual(TT.get(dic, "/ ")[0], 7)
        self.assertEqual(TT.get(dic, "/m~0n")[0], 8)

    def test_22_json_pointer__array(self):
        dic = dict(a=[1, 2], )
        self.assertEqual(TT.get(dic, "/a/1"), (2, ''))

        (val, msg) = TT.get(dic, "/a/2")
        self.assertTrue(val is None)
        self.assertTrue(bool(msg))
        # maybe the error message depends on python version.
        # self.assertEqual(msg, 'list index out of range')

        (val, msg) = TT.get(dic, "/a/b/d/-")
        self.assertTrue(val is None)
        self.assertTrue(bool(msg))
        # Likewise.
        # self.assertEqual(msg, 'list indices must be integers...')


_CNF_0 = OrderedDict((("name", "a"), ("a", 1),
                      ("b", OrderedDict((("b", (1, 2)), ))),
                      ("c", "C"), ("e", [3, 4]), ("f", None)))


class Test_20_make(unittest.TestCase):

    def test_00_null(self):
        md0 = TT.make()

        # check if md0 is an object of base and default class:
        self.assertTrue(isinstance(md0, MD.UpdateWithReplaceDict))
        self.assertTrue(isinstance(md0, MD.UpdateWithMergeDict))
        self.assertTrue(not md0)

    def test_10_default(self):
        md0 = TT.make(_CNF_0)

        self.assertTrue(isinstance(md0, MD.UpdateWithReplaceDict))
        self.assertTrue(isinstance(md0, MD.UpdateWithMergeDict))
        self.assertTrue(isinstance(md0["b"], MD.UpdateWithReplaceDict))
        self.assertTrue(isinstance(md0["b"], MD.UpdateWithMergeDict))
        for k in "name a c e f".split():
            self.assertTrue(md0[k] == _CNF_0[k],
                            "%r vs. %r" % (md0[k], _CNF_0[k]))

    def test_20_merge(self):
        md1 = TT.make(_CNF_0, merge=MG.MS_REPLACE)
        md2 = TT.make(_CNF_0, merge=MG.MS_NO_REPLACE)
        md3 = TT.make(_CNF_0, merge=MG.MS_DICTS_AND_LISTS)

        self.assertTrue(isinstance(md1, MD.UpdateWithReplaceDict))
        self.assertTrue(isinstance(md2, MD.UpdateWoReplaceDict))
        self.assertTrue(isinstance(md3, MD.UpdateWithMergeListsDict))

        for idx, mdn in enumerate((md1, md2, md3)):
            self.assertTrue(isinstance(mdn["b"], type(mdn)),
                            "#%d %r (%r)" % (idx, mdn["b"], type(mdn["b"])))
            for k in "name a c e f".split():
                self.assertTrue(mdn[k] == _CNF_0[k],
                                "%r vs. %r" % (mdn[k], _CNF_0[k]))

        raised = False
        md4 = None
        try:
            md4 = TT.make(_CNF_0, merge="invalid_merge")
        except ValueError:
            raised = True
        self.assertTrue(raised, "md4 = %r" % type(md4))

    def test_30_ordered(self):
        md0 = TT.make(_CNF_0, ordered=True)

        self.assertTrue(isinstance(md0, MD.UpdateWithReplaceDict))
        self.assertTrue(isinstance(md0, MD.UpdateWithMergeOrderedDict))
        self.assertTrue(isinstance(md0["b"], MD.UpdateWithReplaceDict))
        self.assertTrue(isinstance(md0["b"], MD.UpdateWithMergeOrderedDict))
        for k in "name a c e f".split():
            self.assertTrue(md0[k] == _CNF_0[k],
                            "%r vs. %r" % (md0[k], _CNF_0[k]))

    def test_40_namedtuple(self):
        _point = collections.namedtuple("Point", "x y")
        _triangle = collections.namedtuple("Triangle", "p0 p1 p2")
        obj = _triangle(_point(0, 0), _point(1, 0), _point(0, 1))
        md0 = TT.make(obj)

        self.assertTrue(isinstance(md0, MD.UpdateWithMergeOrderedDict))
        self.assertTrue(isinstance(md0["p0"], MD.UpdateWithMergeOrderedDict))

        for k in "p0 p1 p2".split():
            self.assertEqual(md0[MG.NTPL_CLS_KEY], "Triangle")
            for k2 in "x y".split():
                self.assertEqual(md0[k][MG.NTPL_CLS_KEY], "Point")
                ref = getattr(getattr(obj, k), k2)
                self.assertTrue(md0[k][k2] == ref,
                                "%r vs. %r" % (md0[k][k2], ref))


class Test_30_convert_to(unittest.TestCase):

    def test_00_none(self):
        self.assertTrue(TT.convert_to(None) is None)

    def test_10_iterable(self):
        for inp in ([], [0, 1, 2], (), (0, 1), [0, [1, [2]]]):
            self.assertEqual(TT.convert_to(inp), inp)

    def test_20_mdict(self):
        md0 = TT.make(_CNF_0)
        dic0 = TT.convert_to(md0)

        for k in "name a c e f".split():
            self.assertTrue(dic0[k] == md0[k], "%r vs. %r" % (dic0[k], md0[k]))

        for k in dic0["b"].keys():
            self.assertTrue(dic0["b"][k] == md0["b"][k])

    def test_30_to_namedtuple(self):
        _point = collections.namedtuple("Point", "x y")
        _triangle = collections.namedtuple("Triangle", "p0 p1 p2")
        itpl = _triangle(_point(0, 0), _point(1, 0), _point(0, 1))
        md0 = TT.make(itpl)
        otpl = TT.convert_to(md0, to_namedtuple=True)
        self.assertEqual(otpl, itpl)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
"""anyconfig module initialization before any other things.
"""
import logging
import anyconfig.compat

# See: "Configuring Logging for a Library" in python standard logging howto,
# e.g. https://docs.python.org/2/howto/logging.html#library-config.
LOGGER = logging.getLogger("anyconfig")
LOGGER.addHandler(anyconfig.compat.NullHandler())

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Red Hat, Inc.
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato redhat.com>
# License: MIT
#
"""Utility routines.
"""
from __future__ import absolute_import
import collections
import m9dicts.compat


_LIST_LIKE_TYPES = (collections.Iterable, collections.Sequence)


def is_dict_like(obj):
    """
    :param obj: Any object behaves like a dict.

    >>> is_dict_like("a string")
    False
    >>> is_dict_like({})
    True
    >>> is_dict_like(m9dicts.compat.OrderedDict((('a', 1), ('b', 2))))
    True
    """
    return isinstance(obj, (dict, collections.Mapping))  # any others?


def is_namedtuple(obj):
    """
    >>> p0 = collections.namedtuple("Point", "x y")(1, 2)
    >>> is_namedtuple(p0)
    True
    >>> is_namedtuple(tuple(p0))
    False
    """
    return isinstance(obj, tuple) and hasattr(obj, "_asdict")


def is_list_like(obj):
    """
    >>> is_list_like([])
    True
    >>> is_list_like(())
    True
    >>> is_list_like([x for x in range(10)])
    True
    >>> is_list_like((1, 2, 3))
    True
    >>> g = (x for x in range(10))
    >>> is_list_like(g)
    True
    >>> is_list_like("abc")
    False
    >>> is_list_like(0)
    False
    >>> is_list_like({})
    False
    """
    return isinstance(obj, _LIST_LIKE_TYPES) and \
        not (isinstance(obj, m9dicts.compat.STR_TYPES) or is_dict_like(obj))

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2013 - 2016 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
"""configobj backend.

.. versionchanged:: 0.5.0
   Now loading and dumping options are detected automatically from inspection
   result if possible. Also these became not distinguished because these will
   be passed to configobj.Configuration anyway.

- Format to support: configobj, http://goo.gl/JbP2Kp (readthedocs.org)
- Requirements: configobj (https://pypi.python.org/pypi/configobj/)
- Limitations: It seems that configobj.ConfigObj does not receive callble to
  make a dict objects from configurations. If it's true and then the order of
  configurations might be lost.

- Special options:

  - All options except for 'infile' passed to configobj.ConfigObj.__init__
    should work.

  - See also: http://goo.gl/LcVOzZ (readthedocs.org)
"""
from __future__ import absolute_import

import configobj
import inspect
import anyconfig.backend.base


try:
    _LOAD_OPTS = [a for a in inspect.getargspec(configobj.ConfigObj).args
                  if a not in "self infile".split()]
except (TypeError, AttributeError):
    _LOAD_OPTS = ("options configspec encoding interpolation raise_errors"
                  "list_values create_empty file_error stringify"
                  "indent_type default_encoding unrepr write_empty_values"
                  "_inspec").split()


def make_configobj(cnf, **kwargs):
    """
    Make a configobj.ConfigObj initalized with given config `cnf`.

    :param cnf: Configuration data
    :param kwargs: optional keyword parameters passed to ConfigObj.__init__

    :return: An initialized configobj.ConfigObj instance
    """
    cobj = configobj.ConfigObj(**kwargs)
    cobj.update(cnf)

    return cobj


class Parser(anyconfig.backend.base.FromStreamLoader,
             anyconfig.backend.base.ToStreamDumper):
    """
    Parser for Ini-like config files which configobj supports.
    """
    _type = "configobj"
    _priority = 10
    _load_opts = _LOAD_OPTS  # options on dump will be just ignored.
    _dump_opts = _LOAD_OPTS  # Likewise.
    _open_flags = ('rb', 'wb')

    def __load(self, path_or_strm, to_container, **kwargs):
        """
        :param path_or_strm: input config file path or file/file-like object
        """
        return to_container(configobj.ConfigObj(path_or_strm, **kwargs))

    load_from_path = load_from_stream = __load

    def dump_to_string(self, cnf, **kwargs):
        """
        Dump config `cnf` to a string.

        :param cnf: Configuration data to dump
        :param kwargs: backend-specific optional keyword parameters :: dict

        :return: string represents the configuration
        """
        return '\n'.join(make_configobj(cnf, **kwargs).write())

    def dump_to_stream(self, cnf, stream, **kwargs):
        """
        :param cnf: Configuration data to dump
        :param stream: Config file or file-like object
        :param kwargs: backend-specific optional keyword parameters :: dict
        """
        make_configobj(cnf, **kwargs).write(stream)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2013 - 2016 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
"""anyconfig globals.
"""
import anyconfig.init


PACKAGE = "anyconfig"
AUTHOR = "Satoru SATOH <ssat@redhat.com>"
VERSION = "0.7.0"

LOGGER = anyconfig.init.LOGGER

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# Ref. python -c "import json; help(json)"
#
# pylint: disable=import-error
"""JSON file backend.

- Format to support: JSON, http://www.json.org
- Requirements: json in python standard library (>= python 2.6) or simplejson
- Limitations: None obvious
- Special options:

  - All options of json.load{s,} and json.dump{s,} except object_hook
    should work.

  - See also: https://docs.python.org/3/library/json.html or
    https://docs.python.org/2/library/json.html dependent on the python version
    to use.
"""
from __future__ import absolute_import

import anyconfig.backend.base
import anyconfig.compat

try:
    import json
except ImportError:
    import simplejson as json


_LOAD_OPTS = ["cls", "parse_float", "parse_int", "parse_constant"]
_DUMP_OPTS = ["skipkeys", "ensure_ascii", "check_circular", "allow_nan",
              "cls", "indent", "separators", "default", "sort_keys"]

# It seems that 'encoding' argument is not allowed in json.load[s] and
# json.dump[s] in JSON module in python 3.x.
if not anyconfig.compat.IS_PYTHON_3:
    _LOAD_OPTS.append("encoding")
    _DUMP_OPTS.append("encoding")

if not anyconfig.compat.IS_PYTHON_2_6:
    _LOAD_OPTS.append("object_pairs_hook")


class Parser(anyconfig.backend.base.FromStreamLoader,
             anyconfig.backend.base.ToStreamDumper):
    """
    Parser for JSON files.
    """
    _type = "json"
    _extensions = ["json", "jsn", "js"]
    _load_opts = _LOAD_OPTS
    _dump_opts = _DUMP_OPTS

    dump_to_string = anyconfig.backend.base.to_method(json.dumps)
    dump_to_stream = anyconfig.backend.base.to_method(json.dump)

    def _load(self, load_fn, cntnt_or_strm, to_container, **opts):
        """
        Load JSON config from given string or stream `cntnt_or_strm`.

        :param cntnt_or_strm: JSON config content or stream will provide it
        :param to_container: callble to make a container object
        :param opts: keyword options passed to `json.load[s]`

        :return: Dict-like object holding configuration
        """
        if "object_pairs_hook" in self._load_opts:
            opts["object_pairs_hook"] = anyconfig.compat.OrderedDict
            return to_container(load_fn(cntnt_or_strm, **opts))
        else:
            return load_fn(cntnt_or_strm, object_hook=to_container, **opts)

    def load_from_string(self, content, to_container, **opts):
        """
        Load JSON config from given string `content`.

        :param content: JSON config content
        :param to_container: callble to make a container object
        :param opts: keyword options passed to `json.loads`

        :return: Dict-like object holding configuration
        """
        return self._load(json.loads, content, to_container, **opts)

    def load_from_stream(self, stream, to_container, **opts):
        """
        Load JSON config from given stream `stream`.

        :param stream: Stream will provide JSON config content string
        :param to_container: callble to make a container object
        :param opts: keyword options passed to `json.load`

        :return: Dict-like object holding configuration
        """
        return self._load(json.load, stream, to_container, **opts)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Satoru SATOH <ssato @ redhat.com>
#
# pylint: disable=missing-docstring,invalid-name
from __future__ import absolute_import
import unittest
import m9dicts.utils as TT
from m9dicts.compat import OrderedDict


class Test_00_Functions(unittest.TestCase):

    def test_10_is_dict_like(self):
        self.assertFalse(TT.is_dict_like("a string"))
        self.assertTrue(TT.is_dict_like({}))
        self.assertTrue(TT.is_dict_like(OrderedDict((('a', 1), ('b', 2)))))

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2016 Satoru SATOH <ssato redhat.com>
# License: MIT
#
# pylint: disable=unused-argument,unused-import
"""Wrapper of m9dicts.

.. versionadded: 0.5.0
   Swtiched from mergeabledict to m9dicts
"""
from __future__ import absolute_import

import m9dicts
from m9dicts import (
    MS_REPLACE, MS_NO_REPLACE, MS_DICTS, MS_DICTS_AND_LISTS, MERGE_STRATEGIES,
    NTPL_CLS_KEY, UpdateWithReplaceDict, get, set_,
    is_dict_like, is_namedtuple  # flake8: noqa
)


def to_container(obj=None, ac_ordered=False, ac_merge=m9dicts.MS_DICTS,
                 ac_namedtuple=False, ac_ntpl_cls_key=NTPL_CLS_KEY, **options):
    r"""
    Factory function to create a dict-like object[s] supports merge operation
    from a dict or any other objects.

    .. seealso:: :func:`m9dicts.make`

    :param obj: A dict or other object[s] or None
    :param ordered:
        Create an instance of OrderedMergeableDict instead of MergeableDict If
        it's True. Please note that OrderedMergeableDict class will be chosen
        for namedtuple objects regardless of this argument always to keep keys
        (fields) order.
    :param merge:
        Specify strategy from MERGE_STRATEGIES of how to merge results loaded
        from multiple configuration files.
    :param _ntpl_cls_key:
        Special keyword to embedded the class name of namedtuple object to the
        dict-like object created. It's a hack and not elegant but I don't think
        there are another ways to make same namedtuple object from objects
        created from it.
    :param options:
        Optional keyword arguments for m9dicts.convert_to, will be converted to
        the above ac\_\* options respectively as needed.
    """
    opts = dict(ordered=ac_ordered, merge=ac_merge,
                _ntpl_cls_key=ac_ntpl_cls_key)

    return m9dicts.make(obj, **opts)


def convert_to(obj, ac_ordered=True, ac_namedtuple=False,
               ac_ntpl_cls_key=NTPL_CLS_KEY, **options):
    r"""
    Convert given `obj` :: m9dict object to a dict, dict or OrderedDict if
    ac_ordered == True, or a namedtuple if ac_namedtuple == True.

    .. seealso:: :func:`m9dicts.convert_to`

    :param obj: A m9dict object to convert to
    :param ac_ordered: OrderedDict will be chosen if True
    :param ac_namedtuple: A namedtuple object will be chosen if True
    :param ac_ntpl_cls_key: The name of namedtuple object
    :param options:
        Optional keyword arguments for m9dicts.convert_to, will be converted to
        the above ac\_\* options respectively as needed.
    """
    opts = dict(ordered=ac_ordered, to_namedtuple=ac_namedtuple,
                _ntpl_cls_key=ac_ntpl_cls_key)

    return m9dicts.convert_to(obj, **opts)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2012 - 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=unused-import,import-error,invalid-name
r"""Public APIs of anyconfig module.

.. versionadded:: 0.5.0
   - Most keyword arguments passed to APIs are now position independent.
   - Added ac_namedtuple parameter to \*load and \*dump APIs.

.. versionchanged:: 0.3
   Replaced `forced_type` optional argument of some public APIs with
   `ac_parser` to allow skip of config parser search by passing parser object
   previously found and instantiated.

   Also removed some optional arguments, `ignore_missing`, `merge` and
   `marker`, from definitions of some public APIs as these may not be changed
   from default in common use cases.

.. versionchanged:: 0.2
   Now APIs :func:`find_loader`, :func:`single_load`, :func:`multi_load`,
   :func:`load` and :func:`dump` can process a file/file-like object or a list
   of file/file-like objects instead of a file path or a list of file paths.

.. versionadded:: 0.2
   Export factory method (create) of anyconfig.mergeabledict.MergeableDict
"""
from __future__ import absolute_import

import os.path

from anyconfig.globals import LOGGER
import anyconfig.backends
import anyconfig.backend.json
import anyconfig.compat
import anyconfig.mdicts
import anyconfig.template
import anyconfig.utils

# Import some global constants will be re-exported:
from anyconfig.mdicts import (
    MS_REPLACE, MS_NO_REPLACE, MS_DICTS, MS_DICTS_AND_LISTS, MERGE_STRATEGIES,
    get, set_, to_container  # flake8: noqa
)
from anyconfig.schema import validate, gen_schema
from anyconfig.utils import is_path

# Re-export and aliases:
list_types = anyconfig.backends.list_types  # flake8: noqa


def _is_paths(maybe_paths):
    """
    Does given object `maybe_paths` consist of path or path pattern strings?
    """
    if anyconfig.utils.is_iterable(maybe_paths):
        return not getattr(maybe_paths, "read", False)

    return False  # Not an iterable at least.


def _maybe_validated(cnf, schema, format_checker=None, **options):
    """
    :param cnf: Configuration object
    :param schema: JSON schema object
    :param format_checker: A format property checker object of which class is
        inherited from jsonschema.FormatChecker, it's default if None given.
    :param options: Keyword options such as:

        - ac_namedtuple: Convert result to nested namedtuple object if True

    :return: Given `cnf` as it is if validation succeeds else None
    """
    valid = True
    if schema:
        (valid, msg) = validate(cnf, schema, format_checker=format_checker,
                                safe=True)
        if msg:
            LOGGER.warning(msg)

    if valid:
        if options.get("ac_namedtuple", False):
            return anyconfig.mdicts.convert_to(cnf, ac_namedtuple=True)
        else:
            return cnf

    return None


def set_loglevel(level):
    """
    :param level: Log level, e.g. logging.INFO and logging.WARN.
    """
    LOGGER.setLevel(level)


def find_loader(path_or_stream, parser_or_type=None, is_path_=False):
    """
    Find out config parser object appropriate to load from a file of given path
    or file/file-like object.

    :param path_or_stream: Configuration file path or file / file-like object
    :param parser_or_type: Forced configuration parser type or parser object
    :param is_path_: True if given `path_or_stream` is a file path

    :return: Config parser instance or None
    """
    if anyconfig.backends.is_parser(parser_or_type):
        return parser_or_type

    (psr, err) = anyconfig.backends.find_parser(path_or_stream, parser_or_type,
                                                is_path_=is_path_)
    if err:
        LOGGER.error(err)

    if psr is None:
        if parser_or_type is None:
            LOGGER.warning("No parser (type) was given!")
        else:
            LOGGER.warning("Parser %s was not found!", str(parser_or_type))
        return None

    LOGGER.debug("Using config parser: %r [%s]", psr, psr.type())
    return psr()  # TBD: Passing initialization arguments.


def _load_schema(**options):
    """
    :param options: Optional keyword arguments such as

        - ac_template: Assume configuration file may be a template file and try
          to compile it AAR if True
        - ac_context: A dict presents context to instantiate template
        - ac_schema: JSON schema file path to validate given config file
    """
    ac_schema = options.get("ac_schema", None)
    if ac_schema is not None:
        # Try to detect the appropriate as it may be different from the
        # original config file's format, perhaps.
        options["ac_parser"] = None
        options["ac_schema"] = None  # Avoid infinite loop.
        LOGGER.info("Loading schema: %s", ac_schema)
        return load(ac_schema, **options)

    return None


def single_load(path_or_stream, ac_parser=None, ac_template=False,
                ac_context=None, **options):
    """
    Load single config file.

    :param path_or_stream: Configuration file path or file / file-like object
    :param ac_parser: Forced parser type or parser object
    :param ac_template: Assume configuration file may be a template file and
        try to compile it AAR if True
    :param ac_context: A dict presents context to instantiate template
    :param options: Optional keyword arguments such as:

        - Common options:

          - ac_namedtuple: Convert result to nested namedtuple object if True
          - ac_schema: JSON schema file path to validate given config file

        - Common backend options:

          - ignore_missing: Ignore and just return empty result if given file
            (``path_or_stream``) does not exist.

        - Backend specific options such as {"indent": 2} for JSON backend

    :return: dict or dict-like object supports merge operations
    """
    is_path_ = is_path(path_or_stream)
    if is_path_:
        path_or_stream = anyconfig.utils.ensure_expandusr(path_or_stream)
        filepath = path_or_stream
    else:
        filepath = anyconfig.utils.get_path_from_stream(path_or_stream)

    psr = find_loader(path_or_stream, ac_parser, is_path_)
    if psr is None:
        return None

    schema = _load_schema(ac_template=ac_template, ac_context=ac_context,
                          **options)
    options["ac_schema"] = None  # It's not needed now.

    LOGGER.info("Loading: %s", filepath)
    if ac_template and filepath is not None:
        try:
            LOGGER.debug("Compiling: %s", filepath)
            content = anyconfig.template.render(filepath, ac_context)
            cnf = psr.loads(content, **options)
            return _maybe_validated(cnf, schema, **options)

        except Exception as exc:
            LOGGER.warning("Failed to compile %s, fallback to no template "
                           "mode, exc=%r", path_or_stream, exc)

    cnf = psr.load(path_or_stream, **options)
    return _maybe_validated(cnf, schema, **options)


def multi_load(paths, ac_parser=None, ac_template=False, ac_context=None,
               **options):
    """
    Load multiple config files.

    The first argument `paths` may be a list of config file paths or
    a glob pattern specifying that. That is, if a.yml, b.yml and c.yml are in
    the dir /etc/foo/conf.d/, the followings give same results::

      multi_load(["/etc/foo/conf.d/a.yml", "/etc/foo/conf.d/b.yml",
                  "/etc/foo/conf.d/c.yml", ])

      multi_load("/etc/foo/conf.d/*.yml")

    :param paths: List of config file paths or a glob pattern to list paths, or
        a list of file/file-like objects
    :param ac_parser: Forced parser type or parser object
    :param ac_template: Assume configuration file may be a template file and
        try to compile it AAR if True
    :param ac_context: A dict presents context to instantiate template
    :param options: Optional keyword arguments:

        - Common options:

          - ac_merge (merge): Specify strategy of how to merge results loaded
            from multiple configuration files. See the doc of :mod:`m9dicts`
            for more details of strategies. The default is m9dicts.MS_DICTS.

          - ac_marker (marker): Globbing marker to detect paths patterns.
          - ac_namedtuple: Convert result to nested namedtuple object if True
          - ac_schema: JSON schema file path to validate given config file

        - Common backend options:

          - ignore_missing: Ignore and just return empty result if given file
            (``path_or_stream``) does not exist.

        - Backend specific options such as {"indent": 2} for JSON backend

    :return: dict or dict-like object supports merge operations
    """
    marker = options.setdefault("ac_marker", options.get("marker", '*'))
    schema = _load_schema(ac_template=ac_template, ac_context=ac_context,
                          **options)
    options["ac_schema"] = None  # It's not needed now.

    cnf = to_container(ac_context, **options)
    same_type = anyconfig.utils.are_same_file_types(paths)

    if is_path(paths) and marker in paths:
        paths = anyconfig.utils.sglob(paths)

    for path in paths:
        opts = options.copy()
        opts["ac_namedtuple"] = False  # Disabled within this loop.
        # Nested patterns like ['*.yml', '/a/b/c.yml'].
        if is_path(path) and marker in path:
            cups = multi_load(path, ac_parser=ac_parser,
                              ac_template=ac_template, ac_context=cnf, **opts)
        else:
            if same_type:
                ac_parser = find_loader(path, ac_parser, is_path(path))
            cups = single_load(path, ac_parser=ac_parser,
                               ac_template=ac_template, ac_context=cnf, **opts)

        if cups:
            cnf.update(cups)

    return _maybe_validated(cnf, schema, **options)


def load(path_specs, ac_parser=None, ac_template=False, ac_context=None,
         **options):
    r"""
    Load single or multiple config files or multiple config files specified in
    given paths pattern.

    :param path_specs: Configuration file path or paths or its pattern such as
        r'/a/b/\*.json' or a list of files/file-like objects
    :param ac_parser: Forced parser type or parser object
    :param ac_template: Assume configuration file may be a template file and
        try to compile it AAR if True
    :param ac_context: A dict presents context to instantiate template
    :param options:
        Optional keyword arguments. See also the description of `options` in
        `multi_load` function.

    :return: dict or dict-like object supports merge operations
    """
    marker = options.setdefault("ac_marker", options.get("marker", '*'))

    if is_path(path_specs) and marker in path_specs or _is_paths(path_specs):
        return multi_load(path_specs, ac_parser=ac_parser,
                          ac_template=ac_template, ac_context=ac_context,
                          **options)
    else:
        return single_load(path_specs, ac_parser=ac_parser,
                           ac_template=ac_template, ac_context=ac_context,
                           **options)


def loads(content, ac_parser=None, ac_template=False, ac_context=None,
          **options):
    """
    :param content: Configuration file's content
    :param ac_parser: Forced parser type or parser object
    :param ac_template: Assume configuration file may be a template file and
        try to compile it AAR if True
    :param ac_context: Context dict to instantiate template
    :param options:
        Optional keyword arguments. See also the description of `options` in
        `single_load` function.

    :return: dict or dict-like object supports merge operations
    """
    msg = "Try parsing with a built-in parser because %s"
    if ac_parser is None:
        LOGGER.warning(msg, "ac_parser was not given.")
        return None

    psr = find_loader(None, ac_parser)
    if psr is None:
        LOGGER.warning(msg, "parser '%s' was not found" % ac_parser)
        return None

    schema = None
    ac_schema = options.get("ac_schema", None)
    if ac_schema is not None:
        options["ac_schema"] = None
        schema = loads(ac_schema, ac_parser=psr, ac_template=ac_template,
                       ac_context=ac_context, **options)

    if ac_template:
        try:
            LOGGER.debug("Compiling")
            content = anyconfig.template.render_s(content, ac_context)
        except Exception as exc:
            LOGGER.warning("Failed to compile and fallback to no template "
                           "mode: '%s', exc=%r", content[:50] + '...', exc)

    cnf = psr.loads(content, **options)
    return _maybe_validated(cnf, schema, **options)


def _find_dumper(path_or_stream, ac_parser=None):
    """
    Find configuration parser to dump data.

    :param path_or_stream: Output file path or file / file-like object
    :param ac_parser: Forced parser type or parser object

    :return: Parser-inherited class object
    """
    psr = find_loader(path_or_stream, ac_parser)

    if psr is None or not getattr(psr, "dump", False):
        LOGGER.warning("Dump method not implemented. Fallback to json.Parser")
        psr = anyconfig.backend.json.Parser()

    return psr


def dump(data, path_or_stream, ac_parser=None, **options):
    """
    Save `data` as `path_or_stream`.

    :param data: Config data object (dict[-like] or namedtuple) to dump
    :param path_or_stream: Output file path or file / file-like object
    :param ac_parser: Forced parser type or parser object
    :param options:
        Backend specific optional arguments, e.g. {"indent": 2} for JSON
        loader/dumper backend
    """
    dumper = _find_dumper(path_or_stream, ac_parser)
    LOGGER.info("Dumping: %s",
                anyconfig.utils.get_path_from_stream(path_or_stream))
    if anyconfig.mdicts.is_namedtuple(data):
        data = to_container(data, **options)  # namedtuple -> dict-like
    dumper.dump(data, path_or_stream, **options)


def dumps(data, ac_parser=None, **options):
    """
    Return string representation of `data` in forced type format.

    :param data: Config data object to dump
    :param ac_parser: Forced parser type or parser object
    :param options: see :func:`dump`

    :return: Backend-specific string representation for the given data
    """
    if anyconfig.mdicts.is_namedtuple(data):
        data = to_container(data, **options)
    return _find_dumper(None, ac_parser).dumps(data, **options)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2011 - 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
#  pylint: disable=unused-argument
"""INI or INI like config files backend.

.. versionchanged:: 0.3
   Introduce 'ac_parse_value' keyword option to switch behaviors, same as
   original configparser and rich backend parsing each parameter values.

- Format to support: INI or INI like ones
- Requirements: It should be available always.

  - ConfigParser in python 2 standard library:
    https://docs.python.org/2.7/library/configparser.html

  - configparser in python 3 standard library:
    https://docs.python.org/3/library/configparser.html

- Limitations: None obvious
- Special options:

  - Use 'ac_parse_value' boolean keyword option if you want to parse values by
    custom parser, anyconfig.backend.ini._parse.
"""
from __future__ import absolute_import

import anyconfig.backend.base
import anyconfig.parser as P
import anyconfig.utils

from anyconfig.compat import configparser, iteritems
from anyconfig.backend.base import mk_opt_args


_SEP = ','


def _noop(val, *args, **kwargs):
    """
    Parser does nothing.
    """
    # It means nothing but can suppress 'Unused argument' pylint warns.
    # (val, args, kwargs)[0]
    return val


def _parse(val_s, sep=_SEP):
    """
    FIXME: May be too naive implementation.

    :param val_s: A string represents some value to parse
    :param sep: separator between values

    >>> _parse(r'"foo string"')
    'foo string'
    >>> _parse("a, b, c")
    ['a', 'b', 'c']
    >>> _parse("aaa")
    'aaa'
    """
    if (val_s.startswith('"') and val_s.endswith('"')) or \
            (val_s.startswith("'") and val_s.endswith("'")):
        return val_s[1:-1]
    elif sep in val_s:
        return [P.parse(x) for x in P.parse_list(val_s)]
    else:
        return P.parse(val_s)


def _to_s(val, sep=", "):
    """Convert any to string.

    :param val: An object
    :param sep: separator between values

    >>> _to_s([1, 2, 3])
    '1, 2, 3'
    >>> _to_s("aaa")
    'aaa'
    """
    if anyconfig.utils.is_iterable(val):
        return sep.join(str(x) for x in val)
    else:
        return str(val)


def _load(stream, to_container=dict, sep=_SEP, **kwargs):
    """
    :param stream: File or file-like object provides ini-style conf
    :param to_container: any callable to make container
    :param sep: Seprator string

    :return: Dict or dict-like object represents config values
    """
    _parse_val = _parse if kwargs.get("ac_parse_value", False) else _noop

    # Optional arguements for configparser.SafeConfigParser{,readfp}
    kwargs_0 = mk_opt_args(("defaults", "dict_type", "allow_no_value"), kwargs)
    kwargs_1 = mk_opt_args(("filename", ), kwargs)

    try:
        parser = configparser.SafeConfigParser(**kwargs_0)
    except TypeError:
        # .. note::
        #    It seems ConfigPaser.*ConfigParser in python 2.6 does not support
        #    'allow_no_value' option parameter, and TypeError will be thrown.
        kwargs_0 = mk_opt_args(("defaults", "dict_type"), kwargs)
        parser = configparser.SafeConfigParser(**kwargs_0)

    cnf = to_container()
    parser.readfp(stream, **kwargs_1)

    # .. note:: Process DEFAULT config parameters as special ones.
    defaults = parser.defaults()
    if defaults:
        cnf["DEFAULT"] = to_container()
        for key, val in iteritems(defaults):
            cnf["DEFAULT"][key] = _parse_val(val, sep)

    for sect in parser.sections():
        cnf[sect] = to_container()
        for key, val in parser.items(sect):
            cnf[sect][key] = _parse_val(val, sep)

    return cnf


def _dumps_itr(cnf):
    """
    :param cnf: Configuration data to dump
    """
    dkey = "DEFAULT"
    for sect, params in iteritems(cnf):
        yield "[%s]" % sect

        for key, val in iteritems(params):
            if sect != dkey and dkey in cnf and cnf[dkey].get(key) == val:
                continue  # It should be in [DEFAULT] section.

            yield "%s = %s" % (key, _to_s(val))

        yield ''  # it will be a separator between each sections.


def _dumps(cnf, **kwargs):
    """
    :param cnf: Configuration data to dump
    :param kwargs: optional keyword parameters to be sanitized :: dict

    :return: String representation of `cnf` object in INI format
    """
    return '\n'.join(l for l in _dumps_itr(cnf))


class Parser(anyconfig.backend.base.FromStreamLoader,
             anyconfig.backend.base.ToStringDumper):
    """
    Ini config files parser.
    """
    _type = "ini"
    _extensions = ["ini"]
    _load_opts = ["defaults", "dict_type", "allow_no_value", "filename",
                  "ac_parse_value"]

    dump_to_string = anyconfig.backend.base.to_method(_dumps)

    def load_from_stream(self, stream, to_container, **options):
        """
        Load config from given file like object `stream`.

        :param stream:  Config file or file like object
        :param to_container: callble to make a container object
        :param options: optional keyword arguments

        :return: Dict-like object holding config parameters
        """
        return _load(stream, to_container=to_container, **options)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2015, 2016 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# Ref. python -c "import bson; help(bson)"
#
# Access to bson._use_c is required to switch loading options:
# pylint: disable=protected-access
r"""BSON backend.

.. versionchanged:: 0.5.0

   - utilize as_class keyword argument to allow container objects made directly
     on load if C extension is not used and enabled.
   - _load_opts() was removed because C extension looks forced to be enalbed
     if bson.has_c() == True, that is, C extension was built. see also:
     https://jira.mongodb.org/browse/PYTHON-379

.. versionadded:: 0.1.0

- Format to support: BSON, http://bsonspec.org
- Requirements: bson in pymongo, https://pypi.python.org/pypi/pymongo/
- Limitations: It seems that the APIs of bson.decode\* were changed a lot in
  the current version (3.2) of python-bson in pymongo and this backend might
  not work with it. I don't have a time to test with that latest version yet
  and it's only tested with the older one, 2.5.2.
- Special options:

  - All keyword options for :meth:`encode` (dump{s,}) and :meth:`decode`
    (load{s,}) of :class:`bson.BSON` except for as_class should just work.

  - See also: https://api.mongodb.org/python/current/api/bson/
"""
from __future__ import absolute_import

import bson
import anyconfig.backend.base


class Parser(anyconfig.backend.base.FromStringLoader,
             anyconfig.backend.base.ToStringDumper):
    """
    Loader/Dumper of BSON files.
    """
    _type = "bson"
    _extensions = ["bson", "bsn"]  # Temporary.
    _load_opts = [] if bson.has_c() else ["tz_aware", "uuid_subtype"]
    _dump_opts = ["check_keys", "uuid_subtype"]
    _open_flags = ('rb', 'wb')

    dump_to_string = anyconfig.backend.base.to_method(bson.BSON.encode)

    def load_from_string(self, content, to_container, **kwargs):
        """
        Load BSON config from given string `content`.

        :param content: BSON config content in bytes data string
        :param to_container: callble to make a container object
        :param kwargs: optional keyword parameters

        :return: Dict-like object holding config parameters
        """
        if self._load_opts:  # indicates that C extension is not used.
            objs = bson.decode_all(content, as_class=to_container, **kwargs)
        else:
            # .. note::
            #    The order of loaded configuration keys may be lost but
            #    there is no way to avoid that, AFAIK.
            objs = to_container(bson.decode_all(content))

        return objs[0] if objs else None

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=missing-docstring
import anyconfig.backend.bson as TT
import anyconfig.backend.tests.ini

from anyconfig.compat import OrderedDict as ODict, IS_PYTHON_3
from anyconfig.tests.common import to_bytes as _bytes


CNF_0 = ODict((("a", 0.1), ("b", _bytes("bbb")),
               ("sect0",
                ODict((("c", [_bytes("x"), _bytes("y"), _bytes("z")]), )))))


class Test10(anyconfig.backend.tests.ini.Test10):

    cnf = CNF_0
    cnf_s = TT.bson.BSON.encode(CNF_0)
    load_options = dict(as_class=dict)
    dump_options = dict(check_keys=True)

    if IS_PYTHON_3:
        is_order_kept = False  # FIXME: Make it work w/ python 3.

    def setUp(self):
        self.psr = TT.Parser()


class Test20(anyconfig.backend.tests.ini.Test20):

    psr_cls = TT.Parser
    cnf = CNF_0
    cnf_s = TT.bson.BSON.encode(CNF_0)
    cnf_fn = "conf0.bson"

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=missing-docstring
from __future__ import absolute_import

import copy

import anyconfig.backend.msgpack as TT
import anyconfig.backend.tests.ini

from anyconfig.compat import OrderedDict as ODict, IS_PYTHON_3
from anyconfig.tests.common import dicts_equal, to_bytes as _bytes


CNF_0 = ODict(((_bytes("a"), 0.1),
               (_bytes("b"), _bytes("bbb")),
               (_bytes("sect0"),
                ODict(((_bytes("c"),
                        [_bytes("x"), _bytes("y"), _bytes("z")]), )))))


class Test10(anyconfig.backend.tests.ini.Test10):

    cnf = CNF_0
    cnf_s = TT.msgpack.packb(CNF_0)

    if IS_PYTHON_3:
        is_order_kept = False  # FIXME: Make it work w/ python 3.

    def setUp(self):
        self.psr = TT.Parser()

    def test_12_loads__w_options(self):
        cnf = self.psr.loads(self.cnf_s, use_list=False)
        ref = copy.deepcopy(self.cnf)
        ref[_bytes("sect0")][_bytes("c")] = (_bytes("x"), _bytes("y"),
                                             _bytes("z"))
        self.assertTrue(dicts_equal(cnf, ref), str(cnf))

    def test_22_dumps__w_options(self):
        cnf = self.psr.loads(self.psr.dumps(self.cnf, use_single_float=True))
        ref = copy.deepcopy(self.cnf)
        ref[_bytes("a")] = cnf[_bytes("a")]  # single float value.
        self.assertFalse(dicts_equal(cnf, self.cnf), str(cnf))
        self.assertTrue(dicts_equal(cnf, ref), str(cnf))


class Test20(anyconfig.backend.tests.ini.Test20):

    psr_cls = TT.Parser
    cnf = CNF_0
    cnf_s = TT.msgpack.packb(CNF_0)
    cnf_fn = "conf0.msgpack"

    def test_12_load__w_options(self):
        cnf = self.psr.load(self.cpath, use_list=False)
        ref = copy.deepcopy(self.cnf)
        ref[_bytes("sect0")][_bytes("c")] = (_bytes("x"), _bytes("y"),
                                             _bytes("z"))
        self.assertTrue(dicts_equal(cnf, ref), str(cnf))

    def test_22_dump_w_special_option(self):
        self.psr.dump(self.cnf, self.cpath, use_single_float=True)
        cnf = self.psr.load(self.cpath)
        ref = copy.deepcopy(self.cnf)
        ref[_bytes("a")] = cnf[_bytes("a")]  # single float value.
        self.assertFalse(dicts_equal(cnf, self.cnf), str(cnf))
        self.assertTrue(dicts_equal(cnf, ref), str(cnf))

# vim:sw=4:ts=4:et:



#
# Jinja2 (http://jinja.pocoo.org) based template renderer.
#
# Author: Satoru SATOH <ssato redhat.com>
# License: MIT
#
# pylint: disable=unused-argument,wrong-import-position,wrong-import-order
"""Template rendering module for jinja2-based template config files.
"""
from __future__ import absolute_import

import codecs
import locale
import logging
import os

import anyconfig.compat

LOGGER = logging.getLogger(__name__)
SUPPORTED = False
try:
    import jinja2
    from jinja2.exceptions import TemplateNotFound

    SUPPORTED = True

    def tmpl_env(paths):
        """
        :param paths: A list of template search paths
        """
        return jinja2.Environment(loader=jinja2.FileSystemLoader(paths))

except ImportError:
    LOGGER.warning("Jinja2 is not available on your system, so "
                   "template support will be disabled.")

    class TemplateNotFound(RuntimeError):
        """Dummy exception"""
        pass

    def tmpl_env(*args):
        """Dummy function"""
        return None


def copen(filepath, flag='r', encoding=None):

    """
    FIXME: How to test this ?

    >>> c = copen(__file__)
    >>> c is not None
    True
    """
    if encoding is None:
        encoding = locale.getdefaultlocale()[1]

    return codecs.open(filepath, flag, encoding)


def make_template_paths(template_file, paths=None):
    """
    Make up a list of template search paths from given `template_file`
    (absolute or relative path to the template file) and/or `paths` (a list of
    template search paths given by user).

    NOTE: User-given `paths` will take higher priority over a dir of
    template_file.

    :param template_file: Absolute or relative path to the template file
    :param paths: A list of template search paths

    >>> make_template_paths("/path/to/a/template")
    ['/path/to/a']
    >>> make_template_paths("/path/to/a/template", ["/tmp"])
    ['/tmp', '/path/to/a']
    >>> os.chdir("/tmp")
    >>> make_template_paths("./path/to/a/template")
    ['/tmp/path/to/a']
    >>> make_template_paths("./path/to/a/template", ["/tmp"])
    ['/tmp', '/tmp/path/to/a']
    """
    tmpldir = os.path.abspath(os.path.dirname(template_file))
    return [tmpldir] if paths is None else paths + [tmpldir]


def render_s(tmpl_s, ctx=None, paths=None):
    """
    Compile and render given template string `tmpl_s` with context `context`.

    :param tmpl_s: Template string
    :param ctx: Context dict needed to instantiate templates
    :param paths: Template search paths

    >>> render_s("aaa") == "aaa"
    True
    >>> s = render_s('a = {{ a }}, b = "{{ b }}"', {'a': 1, 'b': 'bbb'})
    >>> if SUPPORTED:
    ...     assert s == 'a = 1, b = "bbb"'
    """
    if paths is None:
        paths = [os.curdir]

    env = tmpl_env(paths)

    if env is None:
        return tmpl_s

    if ctx is None:
        ctx = {}

    return tmpl_env(paths).from_string(tmpl_s).render(**ctx)


def render_impl(template_file, ctx=None, paths=None):
    """
    :param template_file: Absolute or relative path to the template file
    :param ctx: Context dict needed to instantiate templates
    """
    env = tmpl_env(make_template_paths(template_file, paths))

    if env is None:
        return copen(template_file).read()

    if ctx is None:
        ctx = {}

    return env.get_template(os.path.basename(template_file)).render(**ctx)


def render(filepath, ctx=None, paths=None, ask=False):
    """
    Compile and render template and return the result as a string.

    :param template_file: Absolute or relative path to the template file
    :param ctx: Context dict needed to instantiate templates
    :param paths: Template search paths
    :param ask: Ask user for missing template location if True
    """
    try:
        return render_impl(filepath, ctx, paths)
    except TemplateNotFound as mtmpl:
        if not ask:
            raise RuntimeError("Template Not found: " + str(mtmpl))

        usr_tmpl = anyconfig.compat.raw_input("\n*** Missing template "
                                              "'%s'. Please enter absolute "
                                              "or relative path starting from "
                                              "'.' to the template file: " %
                                              mtmpl)
        usr_tmpl = os.path.normpath(usr_tmpl.strip())
        paths = make_template_paths(usr_tmpl, paths)

        return render_impl(usr_tmpl, ctx, paths)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2012 - 2015 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=missing-docstring
from __future__ import absolute_import

import unittest

import anyconfig.backends as TT
import anyconfig.backend.ini
import anyconfig.backend.json

try:
    import anyconfig.backend.yaml
    YAML_FOUND = True
except ImportError:
    YAML_FOUND = False


class Test(unittest.TestCase):

    def test_10_find_by_file(self):
        ini_cf = "/a/b/c.ini"
        unknown_cf = "/a/b/c.xyz"
        jsn_cfs = ["/a/b/c.jsn", "/a/b/c.json", "/a/b/c.js"]
        yml_cfs = ["/a/b/c.yml", "/a/b/c.yaml"]

        self.assertTrue(TT.find_by_file(unknown_cf) is None)
        self.assertEqual(TT.find_by_file(ini_cf), anyconfig.backend.ini.Parser)

        for cfg in jsn_cfs:
            self.assertEqual(TT.find_by_file(cfg),
                             anyconfig.backend.json.Parser)

        if YAML_FOUND:
            for cfg in yml_cfs:
                self.assertEqual(TT.find_by_file(cfg),
                                 anyconfig.backend.yaml.Parser)

    def test_20_find_by_type(self):
        ini_t = "ini"
        jsn_t = "json"
        yml_t = "yaml"
        unknown_t = "unknown_type"

        self.assertTrue(TT.find_by_type(unknown_t) is None)
        self.assertEqual(TT.find_by_type(ini_t), anyconfig.backend.ini.Parser)
        self.assertEqual(TT.find_by_type(jsn_t), anyconfig.backend.json.Parser)

        if YAML_FOUND:
            self.assertEqual(TT.find_by_type(yml_t),
                             anyconfig.backend.yaml.Parser)

    def test_30_list_types(self):
        types = TT.list_types()

        self.assertTrue(isinstance(types, list))
        self.assertTrue(bool(list))  # ensure it's not empty.

# vim:sw=4:ts=4:et:
